@@ -13,6 +13,7 @@
 
 #include "mozilla/Attributes.h"
 #include "mozilla/gfx/Types.h"
+#include "mozilla/StaticPrefs_ui.h"
 #include "nsAtom.h"
 #include "nsGkAtoms.h"
 #include "nsCOMPtr.h"
@@ -21,60 +22,17 @@
 #include "nsBoxFrame.h"
 #include "nsMenuParent.h"
 
-#include "nsITimer.h"
-
 #include "Units.h"
 
 class nsIWidget;
 
 namespace mozilla {
+class PresShell;
 namespace dom {
 class KeyboardEvent;
 }  // namespace dom
 }  // namespace mozilla
 
-// XUL popups can be in several different states. When opening a popup, the
-// state changes as follows:
-//   ePopupClosed - initial state
-//   ePopupShowing - during the period when the popupshowing event fires
-//   ePopupOpening - between the popupshowing event and being visible. Creation
-//                   of the child frames, layout and reflow occurs in this
-//                   state. The popup is stored in the popup manager's list of
-//                   open popups during this state.
-//   ePopupVisible - layout is done and the popup's view and widget are made
-//                   visible. The popup is visible on screen but may be
-//                   transitioning. The popupshown event has not yet fired.
-//   ePopupShown - the popup has been shown and is fully ready. This state is
-//                 assigned just before the popupshown event fires.
-// When closing a popup:
-//   ePopupHidden - during the period when the popuphiding event fires and
-//                  the popup is removed.
-//   ePopupClosed - the popup's widget is made invisible.
-enum nsPopupState {
-  // state when a popup is not open
-  ePopupClosed,
-  // state from when a popup is requested to be shown to after the
-  // popupshowing event has been fired.
-  ePopupShowing,
-  // state while a popup is waiting to be laid out and positioned
-  ePopupPositioning,
-  // state while a popup is open but the widget is not yet visible
-  ePopupOpening,
-  // state while a popup is visible and waiting for the popupshown event
-  ePopupVisible,
-  // state while a popup is open and visible on screen
-  ePopupShown,
-  // state from when a popup is requested to be hidden to when it is closed.
-  ePopupHiding,
-  // state which indicates that the popup was hidden without firing the
-  // popuphiding or popuphidden events. It is used when executing a menu
-  // command because the menu needs to be hidden before the command event
-  // fires, yet the popuphiding and popuphidden events are fired after. This
-  // state can also occur when the popup is removed because the document is
-  // unloaded.
-  ePopupInvisible
-};
-
 enum ConsumeOutsideClicksResult {
   ConsumeOutsideClicks_ParentOnly =
       0,                          // Only consume clicks on the parent anchor
@@ -136,15 +94,15 @@ enum MenuPopupAnchorType {
 #define POPUPPOSITION_HFLIP(v) (v ^ 1)
 #define POPUPPOSITION_VFLIP(v) (v ^ 2)
 
-nsIFrame* NS_NewMenuPopupFrame(nsIPresShell* aPresShell,
-                               nsStyleContext* aContext);
+nsIFrame* NS_NewMenuPopupFrame(mozilla::PresShell* aPresShell,
+                               mozilla::ComputedStyle* aStyle);
 
 class nsView;
 class nsMenuPopupFrame;
 
 // this class is used for dispatching popupshown events asynchronously.
-class nsXULPopupShownEvent : public mozilla::Runnable,
-                             public nsIDOMEventListener {
+class nsXULPopupShownEvent final : public mozilla::Runnable,
+                                   public nsIDOMEventListener {
  public:
   nsXULPopupShownEvent(nsIContent* aPopup, nsPresContext* aPresContext)
       : mozilla::Runnable("nsXULPopupShownEvent"),
@@ -158,7 +116,7 @@ class nsXULPopupShownEvent : public mozilla::Runnable,
   void CancelListener();
 
  protected:
-  virtual ~nsXULPopupShownEvent() {}
+  virtual ~nsXULPopupShownEvent() = default;
 
  private:
   nsCOMPtr<nsIContent> mPopup;
@@ -172,12 +130,15 @@ class nsMenuPopupFrame final : public nsBoxFrame,
   NS_DECL_QUERYFRAME
   NS_DECL_FRAMEARENA_HELPERS(nsMenuPopupFrame)
 
-  explicit nsMenuPopupFrame(nsStyleContext* aContext);
+  explicit nsMenuPopupFrame(ComputedStyle* aStyle, nsPresContext* aPresContext);
+  ~nsMenuPopupFrame();
 
   // nsMenuParent interface
   virtual nsMenuFrame* GetCurrentMenuItem() override;
+  MOZ_CAN_RUN_SCRIPT_BOUNDARY
   NS_IMETHOD SetCurrentMenuItem(nsMenuFrame* aMenuItem) override;
   virtual void CurrentMenuIsBeingDestroyed() override;
+  MOZ_CAN_RUN_SCRIPT_BOUNDARY
   NS_IMETHOD ChangeMenuItem(nsMenuFrame* aMenuItem, bool aSelectFirstItem,
                             bool aFromKey) override;
 
@@ -187,8 +148,9 @@ class nsMenuPopupFrame final : public nsBoxFrame,
   void SetPopupState(nsPopupState aPopupState) { mPopupState = aPopupState; }
 
   NS_IMETHOD SetActive(bool aActiveFlag) override {
+    // We don't care.
     return NS_OK;
-  }  // We don't care.
+  }
   virtual bool IsActive() override { return false; }
   virtual bool IsMenuBar() override { return false; }
 
@@ -219,9 +181,6 @@ class nsMenuPopupFrame final : public nsBoxFrame,
 
   nsIWidget* GetWidget();
 
-  // The dismissal listener gets created and attached to the window.
-  void AttachedDismissalListener();
-
   // Overridden methods
   virtual void Init(nsIContent* aContent, nsContainerFrame* aParent,
                     nsIFrame* aPrevInFlow) override;
@@ -246,32 +205,27 @@ class nsMenuPopupFrame final : public nsBoxFrame,
   void EnsureWidget(bool aRecreate = false);
 
   nsresult CreateWidgetForView(nsView* aView);
-  uint8_t GetShadowStyle();
+  mozilla::StyleWindowShadow GetShadowStyle();
 
-  virtual void SetInitialChildList(ChildListID aListID,
-                                   nsFrameList& aChildList) override;
+  bool IsLeafDynamic() const override;
 
-  virtual bool IsLeafDynamic() const override;
-
-  virtual void UpdateWidgetProperties() override;
+  void DidSetComputedStyle(ComputedStyle* aOldStyle) override;
 
   // layout, position and display the popup as needed
+  MOZ_CAN_RUN_SCRIPT_BOUNDARY
   void LayoutPopup(nsBoxLayoutState& aState, nsIFrame* aParentMenu,
-                   nsIFrame* aAnchor, bool aSizedToPopup);
-
-  nsView* GetRootViewForPopup(nsIFrame* aStartFrame);
+                   bool aSizedToPopup);
 
   // Set the position of the popup either relative to the anchor aAnchorFrame
   // (or the frame for mAnchorContent if aAnchorFrame is null), anchored at a
   // rectangle, or at a specific point if a screen position is set. The popup
   // will be adjusted so that it is on screen. If aIsMove is true, then the
-  // popup is being moved, and should not be flipped. If aNotify is true, then
-  // a popuppositioned event is sent.
+  // popup is being moved, and should not be flipped.
   nsresult SetPopupPosition(nsIFrame* aAnchorFrame, bool aIsMove,
-                            bool aSizedToPopup, bool aNotify);
+                            bool aSizedToPopup);
 
-  bool HasGeneratedChildren() { return mGeneratedChildren; }
-  void SetGeneratedChildren() { mGeneratedChildren = true; }
+  // Force the children to be generated if they have not already been generated.
+  void GenerateFrames();
 
   // called when the Enter key is pressed while the popup is open. This will
   // just pass the call down to the current menu, if any. If a current menu
@@ -290,6 +244,11 @@ class nsMenuPopupFrame final : public nsBoxFrame,
   bool IsVisible() {
     return mPopupState == ePopupVisible || mPopupState == ePopupShown;
   }
+  bool IsVisibleOrShowing() {
+    return IsOpen() || mPopupState == ePopupPositioning ||
+           mPopupState == ePopupShowing;
+  }
+  bool IsNativeMenu() { return mIsNativeMenu; }
 
   // Return true if the popup is for a menulist.
   bool IsMenuList();
@@ -298,6 +257,7 @@ class nsMenuPopupFrame final : public nsBoxFrame,
 
   static nsIContent* GetTriggerContent(nsMenuPopupFrame* aMenuPopupFrame);
   void ClearTriggerContent() { mTriggerContent = nullptr; }
+  void ClearTriggerContentIncludingDocument();
 
   // returns true if the popup is in a content shell, or false for a popup in
   // a chrome shell
@@ -322,9 +282,9 @@ class nsMenuPopupFrame final : public nsBoxFrame,
   void InitializePopupAtScreen(nsIContent* aTriggerContent, int32_t aXPos,
                                int32_t aYPos, bool aIsContextMenu);
 
-  void InitializePopupWithAnchorAlign(nsIContent* aAnchorContent,
-                                      nsAString& aAnchor, nsAString& aAlign,
-                                      int32_t aXPos, int32_t aYPos);
+  // Called if this popup should be displayed as an OS-native context menu.
+  void InitializePopupAsNativeContextMenu(nsIContent* aTriggerContent,
+                                          int32_t aXPos, int32_t aYPos);
 
   // indicate that the popup should be opened
   void ShowPopup(bool aIsContextMenu);
@@ -343,17 +303,17 @@ class nsMenuPopupFrame final : public nsBoxFrame,
 
   void ClearIncrementalString() { mIncrementalString.Truncate(); }
   static bool IsWithinIncrementalTime(DOMTimeStamp time) {
-    return !sTimeoutOfIncrementalSearch ||
-           time - sLastKeyTime <= sTimeoutOfIncrementalSearch;
+    return time - sLastKeyTime <=
+           mozilla::StaticPrefs::ui_menu_incremental_search_timeout();
   }
 
 #ifdef DEBUG_FRAME_DUMP
   virtual nsresult GetFrameName(nsAString& aResult) const override {
-    return MakeFrameName(NS_LITERAL_STRING("MenuPopup"), aResult);
+    return MakeFrameName(u"MenuPopup"_ns, aResult);
   }
 #endif
 
-  void EnsureMenuItemIsVisible(nsMenuFrame* aMenuFrame);
+  MOZ_CAN_RUN_SCRIPT void EnsureMenuItemIsVisible(nsMenuFrame* aMenuFrame);
 
   void ChangeByPage(bool aIsUp);
 
@@ -368,13 +328,11 @@ class nsMenuPopupFrame final : public nsBoxFrame,
 
   bool GetAutoPosition();
   void SetAutoPosition(bool aShouldAutoPosition);
-  void SetConsumeRollupEvent(uint32_t aConsumeMode);
 
   nsIScrollableFrame* GetScrollFrame(nsIFrame* aStart);
 
   void SetOverrideConstraintRect(mozilla::LayoutDeviceIntRect aRect) {
-    mOverrideConstraintRect =
-        ToAppUnits(aRect, PresContext()->AppUnitsPerCSSPixel());
+    mOverrideConstraintRect = ToAppUnits(aRect, mozilla::AppUnitsPerCSSPixel());
   }
 
   // For a popup that should appear anchored at the given rect, determine
@@ -448,6 +406,8 @@ class nsMenuPopupFrame final : public nsBoxFrame,
   // with the new rectangle.
   void CheckForAnchorChange(nsRect& aRect);
 
+  void WillDispatchPopupPositioned() { mPendingPositionedEvent = false; }
+
   // nsIReflowCallback
   virtual bool ReflowFinished() override;
   virtual void ReflowCallbackCanceled() override;
@@ -457,7 +417,7 @@ class nsMenuPopupFrame final : public nsBoxFrame,
   nsPopupLevel PopupLevel(bool aIsNoAutoHide) const;
 
   // redefine to tell the box system not to move the views.
-  virtual uint32_t GetXULLayoutFlags() override;
+  ReflowChildFlags GetXULLayoutFlags() override;
 
   void InitPositionFromAnchorAlign(const nsAString& aAnchor,
                                    const nsAString& aAlign);
@@ -532,8 +492,8 @@ class nsMenuPopupFrame final : public nsBoxFrame,
     return mAnchorContent && mAnchorContent->GetPrimaryFrame()
                ? mAnchorContent->GetPrimaryFrame()
                          ->StyleVisibility()
-                         ->mDirection == NS_STYLE_DIRECTION_RTL
-               : StyleVisibility()->mDirection == NS_STYLE_DIRECTION_RTL;
+                         ->mDirection == mozilla::StyleDirection::Rtl
+               : StyleVisibility()->mDirection == mozilla::StyleDirection::Rtl;
   }
 
   // Create a popup view for this frame. The view is added a child of the root
@@ -554,6 +514,19 @@ class nsMenuPopupFrame final : public nsBoxFrame,
  public:
   bool ShouldFollowAnchor(nsRect& aRect);
 
+  // Returns parent menu widget for submenus that are in the same
+  // frame hierarchy, it's needed for Linux/Wayland which demands
+  // strict popup windows hierarchy.
+  nsIWidget* GetParentMenuWidget();
+#ifdef MOZ_WAYLAND
+  // We need following getters for Wayland for calling gdk_window_move_to_rect
+  nsRect GetAnchorRect() { return mAnchorRect; }
+  int GetPopupAlignment() { return mPopupAlignment; }
+  int GetPopupAnchor() { return mPopupAnchor; }
+  int GetPopupPosition() { return mPosition; }
+  FlipType GetFlipType() { return mFlip; }
+#endif
+
  protected:
   nsString mIncrementalString;  // for incremental typing navigation
 
@@ -585,7 +558,11 @@ class nsMenuPopupFrame final : public nsBoxFrame,
   int32_t mXPos;
   int32_t mYPos;
   nsIntRect mScreenRect;
-
+  // Used for store rectangle which the popup is going to be anchored to,
+  // we need that for Wayland
+#ifdef MOZ_WAYLAND
+  nsRect mAnchorRect;
+#endif
   // If the panel prefers to "slide" rather than resize, then the arrow gets
   // positioned at this offset (along either the x or y axis, depending on
   // mPosition)
@@ -604,13 +581,14 @@ class nsMenuPopupFrame final : public nsBoxFrame,
   int8_t mPopupAnchor;
   int8_t mPosition;
 
-  // One of PopupBoxObject::ROLLUP_DEFAULT/ROLLUP_CONSUME/ROLLUP_NO_CONSUME
-  uint8_t mConsumeRollupEvent;
   FlipType mFlip;  // Whether to flip
 
   struct ReflowCallbackData {
     ReflowCallbackData()
-        : mPosted(false), mAnchor(nullptr), mSizedToPopup(false) {}
+        : mPosted(false),
+          mAnchor(nullptr),
+          mSizedToPopup(false),
+          mIsOpenChanged(false) {}
     void MarkPosted(nsIFrame* aAnchor, bool aSizedToPopup,
                     bool aIsOpenChanged) {
       mPosted = true;
@@ -654,6 +632,13 @@ class nsMenuPopupFrame final : public nsBoxFrame,
   bool mHFlip;
   bool mVFlip;
 
+  // Whether the most recent initialization of this menupopup happened via
+  // InitializePopupAsNativeContextMenu.
+  bool mIsNativeMenu = false;
+
+  // Whether we have a pending `popuppositioned` event.
+  bool mPendingPositionedEvent = false;
+
   // When POPUPPOSITION_SELECTION is used, this indicates the vertical offset
   // that the original selected item was. This needs to be used in case the
   // popup gets changed so that we can keep the popup at the same vertical
@@ -669,8 +654,6 @@ class nsMenuPopupFrame final : public nsBoxFrame,
 
   static DOMTimeStamp sLastKeyTime;
 
-  // If 0, never timed out.  Otherwise, the value is in milliseconds.
-  static uint32_t sTimeoutOfIncrementalSearch;
 };  // class nsMenuPopupFrame
 
 #endif
