@@ -13,20 +13,15 @@
 
 #include "nsSliderFrame.h"
 
-#include "gfxPrefs.h"
-#include "nsStyleContext.h"
+#include "mozilla/ComputedStyle.h"
 #include "nsPresContext.h"
 #include "nsIContent.h"
 #include "nsCOMPtr.h"
 #include "nsNameSpaceManager.h"
 #include "nsGkAtoms.h"
 #include "nsHTMLParts.h"
-#include "nsIPresShell.h"
 #include "nsCSSRendering.h"
-#include "nsIDOMEvent.h"
-#include "nsIDOMMouseEvent.h"
 #include "nsScrollbarButtonFrame.h"
-#include "nsISliderListener.h"
 #include "nsIScrollableFrame.h"
 #include "nsIScrollbarMediator.h"
 #include "nsISupportsImpl.h"
@@ -34,28 +29,35 @@
 #include "nsRepeatService.h"
 #include "nsBoxLayoutState.h"
 #include "nsSprocketLayout.h"
-#include "nsIServiceManager.h"
 #include "nsContentUtils.h"
 #include "nsLayoutUtils.h"
 #include "nsDisplayList.h"
-#include "nsRefreshDriver.h"  // for nsAPostRefreshObserver
-#include "nsSVGIntegrationUtils.h"
+#include "nsDeviceContext.h"
+#include "nsRefreshDriver.h"     // for nsAPostRefreshObserver
 #include "mozilla/Assertions.h"  // for MOZ_ASSERT
-#include "mozilla/Preferences.h"
+#include "mozilla/DisplayPortUtils.h"
 #include "mozilla/LookAndFeel.h"
 #include "mozilla/MouseEvents.h"
+#include "mozilla/Preferences.h"
+#include "mozilla/PresShell.h"
+#include "mozilla/SVGIntegrationUtils.h"
 #include "mozilla/Telemetry.h"
+#include "mozilla/dom/Document.h"
+#include "mozilla/dom/Event.h"
 #include "mozilla/layers/APZCCallbackHelper.h"
 #include "mozilla/layers/AsyncDragMetrics.h"
 #include "mozilla/layers/InputAPZContext.h"
+#include "mozilla/layers/WebRenderLayerManager.h"
 #include <algorithm>
 
 using namespace mozilla;
-using mozilla::layers::APZCCallbackHelper;
+using namespace mozilla::gfx;
+using mozilla::dom::Document;
+using mozilla::dom::Event;
 using mozilla::layers::AsyncDragMetrics;
 using mozilla::layers::InputAPZContext;
+using mozilla::layers::ScrollbarData;
 using mozilla::layers::ScrollDirection;
-using mozilla::layers::ScrollThumbData;
 
 bool nsSliderFrame::gMiddlePref = false;
 int32_t nsSliderFrame::gSnapMultiplier;
@@ -68,19 +70,18 @@ static already_AddRefed<nsIContent> GetContentOfBox(nsIFrame* aBox) {
   return content.forget();
 }
 
-nsIFrame* NS_NewSliderFrame(nsIPresShell* aPresShell,
-                            nsStyleContext* aContext) {
-  return new (aPresShell) nsSliderFrame(aContext);
+nsIFrame* NS_NewSliderFrame(PresShell* aPresShell, ComputedStyle* aStyle) {
+  return new (aPresShell) nsSliderFrame(aStyle, aPresShell->GetPresContext());
 }
 
 NS_IMPL_FRAMEARENA_HELPERS(nsSliderFrame)
 
 NS_QUERYFRAME_HEAD(nsSliderFrame)
-NS_QUERYFRAME_ENTRY(nsSliderFrame)
+  NS_QUERYFRAME_ENTRY(nsSliderFrame)
 NS_QUERYFRAME_TAIL_INHERITING(nsBoxFrame)
 
-nsSliderFrame::nsSliderFrame(nsStyleContext* aContext)
-    : nsBoxFrame(aContext, kClassID),
+nsSliderFrame::nsSliderFrame(ComputedStyle* aStyle, nsPresContext* aPresContext)
+    : nsBoxFrame(aStyle, aPresContext, kClassID),
       mRatio(0.0f),
       mDragStart(0),
       mThumbStart(0),
@@ -94,8 +95,9 @@ nsSliderFrame::nsSliderFrame(nsStyleContext* aContext)
 // stop timer
 nsSliderFrame::~nsSliderFrame() {
   if (mSuppressionActive) {
-    APZCCallbackHelper::SuppressDisplayport(
-        false, PresContext() ? PresShell() : nullptr);
+    if (mozilla::PresShell* presShell = PresShell()) {
+      presShell->SuppressDisplayport(false);
+    }
   }
 }
 
@@ -120,9 +122,10 @@ void nsSliderFrame::RemoveFrame(ChildListID aListID, nsIFrame* aOldFrame) {
 }
 
 void nsSliderFrame::InsertFrames(ChildListID aListID, nsIFrame* aPrevFrame,
+                                 const nsLineList::iterator* aPrevFrameLine,
                                  nsFrameList& aFrameList) {
   bool wasEmpty = mFrames.IsEmpty();
-  nsBoxFrame::InsertFrames(aListID, aPrevFrame, aFrameList);
+  nsBoxFrame::InsertFrames(aListID, aPrevFrame, aPrevFrameLine, aFrameList);
   if (wasEmpty) AddListener();
 }
 
@@ -170,42 +173,6 @@ int32_t nsSliderFrame::GetIntegerAttribute(nsIContent* content, nsAtom* atom,
   return defaultValue;
 }
 
-class nsValueChangedRunnable : public Runnable {
- public:
-  nsValueChangedRunnable(nsISliderListener* aListener, nsAtom* aWhich,
-                         int32_t aValue, bool aUserChanged)
-      : mozilla::Runnable("nsValueChangedRunnable"),
-        mListener(aListener),
-        mWhich(aWhich),
-        mValue(aValue),
-        mUserChanged(aUserChanged) {}
-
-  NS_IMETHOD Run() override {
-    return mListener->ValueChanged(nsDependentAtomString(mWhich), mValue,
-                                   mUserChanged);
-  }
-
-  nsCOMPtr<nsISliderListener> mListener;
-  RefPtr<nsAtom> mWhich;
-  int32_t mValue;
-  bool mUserChanged;
-};
-
-class nsDragStateChangedRunnable : public Runnable {
- public:
-  nsDragStateChangedRunnable(nsISliderListener* aListener, bool aDragBeginning)
-      : mozilla::Runnable("nsDragStateChangedRunnable"),
-        mListener(aListener),
-        mDragBeginning(aDragBeginning) {}
-
-  NS_IMETHOD Run() override {
-    return mListener->DragStateChanged(mDragBeginning);
-  }
-
-  nsCOMPtr<nsISliderListener> mListener;
-  bool mDragBeginning;
-};
-
 nsresult nsSliderFrame::AttributeChanged(int32_t aNameSpaceID,
                                          nsAtom* aAttribute, int32_t aModType) {
   nsresult rv =
@@ -223,18 +190,6 @@ nsresult nsSliderFrame::AttributeChanged(int32_t aNameSpaceID,
     int32_t min = GetMinPosition(scrollbar);
     int32_t max = GetMaxPosition(scrollbar);
 
-    // inform the parent <scale> that the minimum or maximum changed
-    nsIFrame* parent = GetParent();
-    if (parent) {
-      nsCOMPtr<nsISliderListener> sliderListener =
-          do_QueryInterface(parent->GetContent());
-      if (sliderListener) {
-        nsContentUtils::AddScriptRunner(new nsValueChangedRunnable(
-            sliderListener, aAttribute,
-            aAttribute == nsGkAtoms::minpos ? min : max, false));
-      }
-    }
-
     if (current < min || current > max) {
       int32_t direction = 0;
       if (current < min || max < min) {
@@ -265,24 +220,173 @@ nsresult nsSliderFrame::AttributeChanged(int32_t aNameSpaceID,
   if (aAttribute == nsGkAtoms::minpos || aAttribute == nsGkAtoms::maxpos ||
       aAttribute == nsGkAtoms::pageincrement ||
       aAttribute == nsGkAtoms::increment) {
-    PresShell()->FrameNeedsReflow(this, nsIPresShell::eStyleChange,
+    PresShell()->FrameNeedsReflow(this, IntrinsicDirty::StyleChange,
                                   NS_FRAME_IS_DIRTY);
   }
 
   return rv;
 }
 
+// Draw any tick marks that show the position of find in page results.
+class nsDisplaySliderMarks final : public nsPaintedDisplayItem {
+ public:
+  nsDisplaySliderMarks(nsDisplayListBuilder* aBuilder, nsSliderFrame* aFrame)
+      : nsPaintedDisplayItem(aBuilder, aFrame) {
+    MOZ_COUNT_CTOR(nsDisplaySliderMarks);
+  }
+  MOZ_COUNTED_DTOR_OVERRIDE(nsDisplaySliderMarks)
+
+  NS_DISPLAY_DECL_NAME("SliderMarks", TYPE_SLIDER_MARKS)
+
+  void PaintMarks(nsDisplayListBuilder* aDisplayListBuilder,
+                  wr::DisplayListBuilder* aBuilder, gfxContext* aCtx);
+
+  virtual nsRect GetBounds(nsDisplayListBuilder* aBuilder,
+                           bool* aSnap) const override {
+    *aSnap = false;
+    return mFrame->InkOverflowRectRelativeToSelf() + ToReferenceFrame();
+  }
+
+  bool CreateWebRenderCommands(
+      wr::DisplayListBuilder& aBuilder, wr::IpcResourceUpdateQueue& aResources,
+      const StackingContextHelper& aSc,
+      mozilla::layers::RenderRootStateManager* aManager,
+      nsDisplayListBuilder* aDisplayListBuilder) override;
+
+  void Paint(nsDisplayListBuilder* aBuilder, gfxContext* aCtx) override;
+};
+
+// This is shared between the webrender and Paint() paths. For the former,
+// aBuilder should be assigned and aCtx will be null. For the latter, aBuilder
+// should be null and aCtx should be the gfxContext for painting.
+void nsDisplaySliderMarks::PaintMarks(nsDisplayListBuilder* aDisplayListBuilder,
+                                      wr::DisplayListBuilder* aBuilder,
+                                      gfxContext* aCtx) {
+  DrawTarget* drawTarget = nullptr;
+  if (aCtx) {
+    drawTarget = aCtx->GetDrawTarget();
+  } else {
+    MOZ_ASSERT(aBuilder);
+  }
+
+  Document* doc = mFrame->GetContent()->GetUncomposedDoc();
+  if (!doc) {
+    return;
+  }
+
+  nsGlobalWindowInner* window =
+      nsGlobalWindowInner::Cast(doc->GetInnerWindow());
+  if (!window) {
+    return;
+  }
+
+  nsSliderFrame* sliderFrame = static_cast<nsSliderFrame*>(mFrame);
+
+  nsIFrame* scrollbarBox = sliderFrame->GetScrollbar();
+  nsCOMPtr<nsIContent> scrollbar = GetContentOfBox(scrollbarBox);
+
+  int32_t minPos = sliderFrame->GetMinPosition(scrollbar);
+  int32_t maxPos = sliderFrame->GetMaxPosition(scrollbar);
+
+  // Use the text highlight color for the tick marks.
+  nscolor highlightColor =
+      LookAndFeel::Color(LookAndFeel::ColorID::TextHighlightBackground, mFrame);
+  DeviceColor fillColor = ToDeviceColor(highlightColor);
+  fillColor.a = 0.3;  // make the mark mostly transparent
+
+  int32_t appUnitsPerDevPixel =
+      sliderFrame->PresContext()->AppUnitsPerDevPixel();
+  nsRect sliderRect = sliderFrame->GetRect();
+
+  nsPoint refPoint = aDisplayListBuilder->ToReferenceFrame(mFrame);
+
+  // Increase the height of the tick mark rectangle by one pixel. If the
+  // desktop scale is greater than 1, it should be increased more.
+  // The tick marks should be drawn ignoring any page zoom that is applied.
+  float increasePixels = sliderFrame->PresContext()
+                             ->DeviceContext()
+                             ->GetDesktopToDeviceScale()
+                             .scale;
+
+  nsTArray<uint32_t>& marks = window->GetScrollMarks();
+  for (uint32_t m = 0; m < marks.Length(); m++) {
+    uint32_t markValue = marks[m];
+    if (markValue > (uint32_t)maxPos) {
+      markValue = maxPos;
+    }
+    if (markValue < (uint32_t)minPos) {
+      markValue = minPos;
+    }
+
+    // The values in the marks array range up to the window's scrollMaxY
+    // (the same as the slider's maxpos). Scale the values to fit within
+    // the slider's height.
+    nsRect markRect(refPoint, nsSize(sliderRect.width, 0));
+    markRect.y +=
+        (nscoord)((double)markValue / (maxPos - minPos) * sliderRect.height);
+
+    if (drawTarget) {
+      Rect devPixelRect =
+          NSRectToSnappedRect(markRect, appUnitsPerDevPixel, *drawTarget);
+      devPixelRect.Inflate(0, increasePixels);
+      drawTarget->FillRect(devPixelRect, ColorPattern(fillColor));
+    } else {
+      LayoutDeviceIntRect dRect = LayoutDeviceIntRect::FromAppUnitsToNearest(
+          markRect, appUnitsPerDevPixel);
+      dRect.Inflate(0, increasePixels);
+      wr::LayoutRect layoutRect = wr::ToLayoutRect(dRect);
+      aBuilder->PushRect(layoutRect, layoutRect, BackfaceIsHidden(),
+                         wr::ToColorF(fillColor));
+    }
+  }
+}
+
+bool nsDisplaySliderMarks::CreateWebRenderCommands(
+    wr::DisplayListBuilder& aBuilder, wr::IpcResourceUpdateQueue& aResources,
+    const StackingContextHelper& aSc,
+    mozilla::layers::RenderRootStateManager* aManager,
+    nsDisplayListBuilder* aDisplayListBuilder) {
+  PaintMarks(aDisplayListBuilder, &aBuilder, nullptr);
+  return true;
+}
+
+void nsDisplaySliderMarks::Paint(nsDisplayListBuilder* aBuilder,
+                                 gfxContext* aCtx) {
+  PaintMarks(aBuilder, nullptr, aCtx);
+}
+
 void nsSliderFrame::BuildDisplayList(nsDisplayListBuilder* aBuilder,
                                      const nsDisplayListSet& aLists) {
   if (aBuilder->IsForEventDelivery() && isDraggingThumb()) {
     // This is EVIL, we shouldn't be messing with event delivery just to get
     // thumb mouse drag events to arrive at the slider!
-    aLists.Outlines()->AppendToTop(
-        MakeDisplayItem<nsDisplayEventReceiver>(aBuilder, this));
+    aLists.Outlines()->AppendNewToTop<nsDisplayEventReceiver>(aBuilder, this);
     return;
   }
 
   nsBoxFrame::BuildDisplayList(aBuilder, aLists);
+
+  // If this is a vertical scrollbar for the root frame, draw any markers.
+  // Markers are not drawn for other scrollbars.
+  if (!aBuilder->IsForEventDelivery()) {
+    nsIFrame* scrollbarBox = GetScrollbar();
+    if (scrollbarBox && !IsXULHorizontal()) {
+      if (nsIScrollableFrame* scrollFrame =
+              do_QueryFrame(scrollbarBox->GetParent())) {
+        if (scrollFrame->IsRootScrollFrameOfDocument()) {
+          Document* doc = mContent->GetUncomposedDoc();
+          if (doc) {
+            nsGlobalWindowInner* window =
+                nsGlobalWindowInner::Cast(doc->GetInnerWindow());
+            if (window && window->GetScrollMarks().Length() > 0) {
+              aLists.Content()->AppendNewToTop<nsDisplaySliderMarks>(aBuilder,
+                                                                     this);
+            }
+          }
+        }
+      }
+    }
+  }
 }
 
 static bool UsesCustomScrollbarMediator(nsIFrame* scrollbarBox) {
@@ -303,7 +407,7 @@ static bool UsesCustomScrollbarMediator(nsIFrame* scrollbarBox) {
 void nsSliderFrame::BuildDisplayListForChildren(
     nsDisplayListBuilder* aBuilder, const nsDisplayListSet& aLists) {
   // if we are too small to have a thumb don't paint it.
-  nsIFrame* thumb = nsBox::GetChildXULBox(this);
+  nsIFrame* thumb = nsIFrame::GetChildXULBox(this);
 
   if (thumb) {
     nsRect thumbRect(thumb->GetRect());
@@ -325,22 +429,19 @@ void nsSliderFrame::BuildDisplayListForChildren(
     // that the event region that gets created for the thumb is included in
     // the nsDisplayOwnLayer contents.
 
-    nsDisplayOwnLayerFlags flags = aBuilder->GetCurrentScrollbarFlags();
-    mozilla::layers::FrameMetrics::ViewID scrollTargetId =
+    const mozilla::layers::ScrollableLayerGuid::ViewID scrollTargetId =
         aBuilder->GetCurrentScrollbarTarget();
-    bool thumbGetsLayer =
-        (scrollTargetId != layers::FrameMetrics::NULL_SCROLL_ID);
+    const bool thumbGetsLayer =
+        (scrollTargetId != layers::ScrollableLayerGuid::NULL_SCROLL_ID);
 
     if (thumbGetsLayer) {
-      MOZ_ASSERT((flags & nsDisplayOwnLayerFlags::eHorizontalScrollbar) ||
-                 (flags & nsDisplayOwnLayerFlags::eVerticalScrollbar));
-      bool isHorizontal =
-          bool(flags & nsDisplayOwnLayerFlags::eHorizontalScrollbar);
-      ScrollDirection scrollDirection = isHorizontal
-                                            ? ScrollDirection::eHorizontal
-                                            : ScrollDirection::eVertical;
+      const Maybe<ScrollDirection> scrollDirection =
+          aBuilder->GetCurrentScrollbarDirection();
+      MOZ_ASSERT(scrollDirection.isSome());
+      const bool isHorizontal =
+          *scrollDirection == ScrollDirection::eHorizontal;
       const float appUnitsPerCss = float(AppUnitsPerCSSPixel());
-      CSSCoord thumbLength = NSAppUnitsToFloatPixels(
+      const CSSCoord thumbLength = NSAppUnitsToFloatPixels(
           isHorizontal ? thumbRect.width : thumbRect.height, appUnitsPerCss);
 
       nsIFrame* scrollbarBox = GetScrollbar();
@@ -357,27 +458,22 @@ void nsSliderFrame::BuildDisplayListForChildren(
       // This rect is the range in which the scroll thumb can slide in.
       sliderTrack = sliderTrack + GetRect().TopLeft() +
                     scrollbarBox->GetPosition() - scrollPortOrigin;
-      CSSCoord sliderTrackStart = NSAppUnitsToFloatPixels(
+      const CSSCoord sliderTrackStart = NSAppUnitsToFloatPixels(
           isHorizontal ? sliderTrack.x : sliderTrack.y, appUnitsPerCss);
-      CSSCoord sliderTrackLength = NSAppUnitsToFloatPixels(
+      const CSSCoord sliderTrackLength = NSAppUnitsToFloatPixels(
           isHorizontal ? sliderTrack.width : sliderTrack.height,
           appUnitsPerCss);
-      CSSCoord thumbStart = NSAppUnitsToFloatPixels(
+      const CSSCoord thumbStart = NSAppUnitsToFloatPixels(
           isHorizontal ? thumbRect.x : thumbRect.y, appUnitsPerCss);
 
-      nsRect overflow = thumb->GetVisualOverflowRectRelativeToParent();
+      const nsRect overflow = thumb->InkOverflowRectRelativeToParent();
       nsSize refSize = aBuilder->RootReferenceFrame()->GetSize();
-      gfxSize scale = nsLayoutUtils::GetTransformToAncestorScale(thumb);
-      if (scale.width != 0 && scale.height != 0) {
-        refSize.width /= scale.width;
-        refSize.height /= scale.height;
-      }
       nsRect dirty = aBuilder->GetVisibleRect().Intersect(thumbRect);
       dirty = nsLayoutUtils::ComputePartialPrerenderArea(
-          aBuilder->GetVisibleRect(), overflow, refSize);
+          thumb, aBuilder->GetVisibleRect(), overflow, refSize);
 
       nsDisplayListBuilder::AutoBuildingDisplayList buildingDisplayList(
-          aBuilder, this, dirty, dirty, false);
+          aBuilder, this, dirty, dirty);
 
       // Clip the thumb layer to the slider track. This is necessary to ensure
       // FrameLayerBuilder is able to merge content before and after the
@@ -413,11 +509,15 @@ void nsSliderFrame::BuildDisplayListForChildren(
 
       // Wrap the list to make it its own layer.
       const ActiveScrolledRoot* ownLayerASR = contASRTracker.GetContainerASR();
-      aLists.Content()->AppendToTop(MakeDisplayItem<nsDisplayOwnLayer>(
-          aBuilder, this, &masterList, ownLayerASR, flags, scrollTargetId,
-          ScrollThumbData{scrollDirection, GetThumbRatio(), thumbStart,
-                          thumbLength, isAsyncDraggable, sliderTrackStart,
-                          sliderTrackLength}));
+      aLists.Content()->AppendNewToTopWithIndex<nsDisplayOwnLayer>(
+          aBuilder, this,
+          /* aIndex = */ nsDisplayOwnLayer::OwnLayerForScrollThumb, &masterList,
+          ownLayerASR, nsDisplayOwnLayerFlags::None,
+          ScrollbarData::CreateForThumb(*scrollDirection, GetThumbRatio(),
+                                        thumbStart, thumbLength,
+                                        isAsyncDraggable, sliderTrackStart,
+                                        sliderTrackLength, scrollTargetId),
+          true, false);
 
       return;
     }
@@ -429,24 +529,15 @@ void nsSliderFrame::BuildDisplayListForChildren(
 NS_IMETHODIMP
 nsSliderFrame::DoXULLayout(nsBoxLayoutState& aState) {
   // get the thumb should be our only child
-  nsIFrame* thumbBox = nsBox::GetChildXULBox(this);
+  nsIFrame* thumbBox = nsIFrame::GetChildXULBox(this);
 
   if (!thumbBox) {
-    SyncLayout(aState);
+    SyncXULLayout(aState);
     return NS_OK;
   }
 
   EnsureOrient();
 
-#ifdef DEBUG_LAYOUT
-  if (mState & NS_STATE_DEBUG_WAS_SET) {
-    if (mState & NS_STATE_SET_TO_DEBUG)
-      SetXULDebug(aState, true);
-    else
-      SetXULDebug(aState, false);
-  }
-#endif
-
   // get the content area inside our borders
   nsRect clientRect;
   GetXULClientRect(clientRect);
@@ -507,7 +598,7 @@ nsSliderFrame::DoXULLayout(nsBoxLayoutState& aState) {
   nsRect oldThumbRect(thumbBox->GetRect());
   LayoutChildAt(aState, thumbBox, thumbRect);
 
-  SyncLayout(aState);
+  SyncXULLayout(aState);
 
   // Redraw only if thumb changed size.
   if (!oldThumbRect.IsEqualInterior(thumbRect)) XULRedraw(aState);
@@ -520,6 +611,22 @@ nsresult nsSliderFrame::HandleEvent(nsPresContext* aPresContext,
                                     nsEventStatus* aEventStatus) {
   NS_ENSURE_ARG_POINTER(aEventStatus);
 
+  if (mAPZDragInitiated &&
+      *mAPZDragInitiated == InputAPZContext::GetInputBlockId() &&
+      aEvent->mMessage == eMouseDown) {
+    // If we get the mousedown after the APZ notification, then immediately
+    // switch into the state corresponding to an APZ thumb-drag. Note that
+    // we can't just do this in AsyncScrollbarDragInitiated() directly because
+    // the handling for this mousedown event in the presShell will reset the
+    // capturing content which makes isDraggingThumb() return false. We check
+    // the input block here to make sure that we correctly handle any ordering
+    // of {eMouseDown arriving, AsyncScrollbarDragInitiated() being called}.
+    mAPZDragInitiated = Nothing();
+    DragThumb(true);
+    mScrollingWithAPZ = true;
+    return NS_OK;
+  }
+
   // If a web page calls event.preventDefault() we still want to
   // scroll when scroll arrow is clicked. See bug 511075.
   if (!mContent->IsInNativeAnonymousSubtree() &&
@@ -549,9 +656,9 @@ nsresult nsSliderFrame::HandleEvent(nsPresContext* aPresContext,
           break;
         }
         if (mChange) {
-        // On Linux the destination point is determined by the initial click
-        // on the scrollbar track and doesn't change until the mouse button
-        // is released.
+          // On Linux the destination point is determined by the initial click
+          // on the scrollbar track and doesn't change until the mouse button
+          // is released.
 #ifndef MOZ_WIDGET_GTK
           // On the other platforms we need to update the destination point now.
           mDestinationPoint = eventPoint;
@@ -607,7 +714,7 @@ nsresult nsSliderFrame::HandleEvent(nsPresContext* aPresContext,
           StopDrag();
           // we MUST call nsFrame HandleEvent for mouse ups to maintain the
           // selection state and capture state.
-          return nsFrame::HandleEvent(aPresContext, aEvent, aEventStatus);
+          return nsIFrame::HandleEvent(aPresContext, aEvent, aEventStatus);
         }
         break;
 
@@ -615,9 +722,11 @@ nsresult nsSliderFrame::HandleEvent(nsPresContext* aPresContext,
         break;
     }
 
-    // return nsFrame::HandleEvent(aPresContext, aEvent, aEventStatus);
+    // return nsIFrame::HandleEvent(aPresContext, aEvent, aEventStatus);
     return NS_OK;
-  } else if (ShouldScrollToClickForEvent(aEvent)) {
+  }
+
+  if (ShouldScrollToClickForEvent(aEvent)) {
     nsPoint eventPoint;
     if (!GetEventPoint(aEvent, eventPoint)) {
       return NS_OK;
@@ -641,9 +750,8 @@ nsresult nsSliderFrame::HandleEvent(nsPresContext* aPresContext,
     DragThumb(true);
 
 #ifdef MOZ_WIDGET_GTK
-    RefPtr<Element> thumb = thumbFrame->GetContent()->AsElement();
-    thumb->SetAttr(kNameSpaceID_None, nsGkAtoms::active,
-                   NS_LITERAL_STRING("true"), true);
+    RefPtr<dom::Element> thumb = thumbFrame->GetContent()->AsElement();
+    thumb->SetAttr(kNameSpaceID_None, nsGkAtoms::active, u"true"_ns, true);
 #endif
 
     if (aEvent->mClass == eTouchEventClass) {
@@ -659,9 +767,9 @@ nsresult nsSliderFrame::HandleEvent(nsPresContext* aPresContext,
   }
 #ifdef MOZ_WIDGET_GTK
   else if (ShouldScrollForEvent(aEvent) && aEvent->mClass == eMouseEventClass &&
-           aEvent->AsMouseEvent()->button == WidgetMouseEvent::eRightButton) {
+           aEvent->AsMouseEvent()->mButton == MouseButton::eSecondary) {
     // HandlePress and HandleRelease are usually called via
-    // nsFrame::HandleEvent, but only for the left mouse button.
+    // nsIFrame::HandleEvent, but only for the left mouse button.
     if (aEvent->mMessage == eMouseDown) {
       HandlePress(aPresContext, aEvent, aEventStatus);
     } else if (aEvent->mMessage == eMouseUp) {
@@ -682,7 +790,7 @@ nsresult nsSliderFrame::HandleEvent(nsPresContext* aPresContext,
   if (aEvent->mMessage == eMouseOut && mChange)
     HandleRelease(aPresContext, aEvent, aEventStatus);
 
-  return nsFrame::HandleEvent(aPresContext, aEvent, aEventStatus);
+  return nsIFrame::HandleEvent(aPresContext, aEvent, aEventStatus);
 }
 
 // Helper function to collect the "scroll to click" metric. Beware of
@@ -690,7 +798,7 @@ nsresult nsSliderFrame::HandleEvent(nsPresContext* aPresContext,
 // and see the browser change its behavior immediately.
 bool nsSliderFrame::GetScrollToClick() {
   if (GetScrollbar() != this) {
-    return LookAndFeel::GetInt(LookAndFeel::eIntID_ScrollToClick, false);
+    return LookAndFeel::GetInt(LookAndFeel::IntID::ScrollToClick, false);
   }
 
   if (mContent->AsElement()->AttrValueIs(kNameSpaceID_None,
@@ -811,27 +919,16 @@ void nsSliderFrame::CurrentPositionChanged() {
   }
 
   mCurPos = curPos;
-
-  // inform the parent <scale> if it exists that the value changed
-  nsIFrame* parent = GetParent();
-  if (parent) {
-    nsCOMPtr<nsISliderListener> sliderListener =
-        do_QueryInterface(parent->GetContent());
-    if (sliderListener) {
-      nsContentUtils::AddScriptRunner(new nsValueChangedRunnable(
-          sliderListener, nsGkAtoms::curpos, mCurPos, mUserChanged));
-    }
-  }
 }
 
-static void UpdateAttribute(Element* aScrollbar, nscoord aNewPos, bool aNotify,
-                            bool aIsSmooth) {
+static void UpdateAttribute(dom::Element* aScrollbar, nscoord aNewPos,
+                            bool aNotify, bool aIsSmooth) {
   nsAutoString str;
   str.AppendInt(aNewPos);
 
   if (aIsSmooth) {
-    aScrollbar->SetAttr(kNameSpaceID_None, nsGkAtoms::smooth,
-                        NS_LITERAL_STRING("true"), false);
+    aScrollbar->SetAttr(kNameSpaceID_None, nsGkAtoms::smooth, u"true"_ns,
+                        false);
   }
   aScrollbar->SetAttr(kNameSpaceID_None, nsGkAtoms::curpos, str, aNotify);
   if (aIsSmooth) {
@@ -938,58 +1035,22 @@ void nsSliderFrame::SetInitialChildList(ChildListID aListID,
   }
 }
 
-nsresult nsSliderMediator::HandleEvent(nsIDOMEvent* aEvent) {
+nsresult nsSliderMediator::HandleEvent(dom::Event* aEvent) {
   // Only process the event if the thumb is not being dragged.
   if (mSlider && !mSlider->isDraggingThumb()) return mSlider->StartDrag(aEvent);
 
   return NS_OK;
 }
 
-class AsyncScrollbarDragStarter : public nsAPostRefreshObserver {
- public:
-  AsyncScrollbarDragStarter(nsIPresShell* aPresShell, nsIWidget* aWidget,
-                            const AsyncDragMetrics& aDragMetrics)
-      : mPresShell(aPresShell), mWidget(aWidget), mDragMetrics(aDragMetrics) {}
-  virtual ~AsyncScrollbarDragStarter() {}
-
-  void DidRefresh() override {
-    if (!mPresShell) {
-      MOZ_ASSERT_UNREACHABLE(
-          "Post-refresh observer fired again after failed attempt at "
-          "unregistering it");
-      return;
-    }
-
-    mWidget->StartAsyncScrollbarDrag(mDragMetrics);
-
-    if (!mPresShell->RemovePostRefreshObserver(this)) {
-      MOZ_ASSERT_UNREACHABLE(
-          "Unable to unregister post-refresh observer! Leaking it instead of "
-          "leaving garbage registered");
-      // Graceful handling, just in case...
-      mPresShell = nullptr;
-      mWidget = nullptr;
-      return;
-    }
-
-    delete this;
-  }
-
- private:
-  RefPtr<nsIPresShell> mPresShell;
-  RefPtr<nsIWidget> mWidget;
-  AsyncDragMetrics mDragMetrics;
-};
-
-static bool UsesSVGEffects(nsIFrame* aFrame) {
-  return aFrame->StyleEffects()->HasFilters() ||
-         nsSVGIntegrationUtils::UsingMaskOrClipPathForFrame(aFrame);
-}
-
 static bool ScrollFrameWillBuildScrollInfoLayer(nsIFrame* aScrollFrame) {
+  /*
+   * Note: if changing the conditions in this function, make a corresponding
+   * change to nsDisplayListBuilder::ShouldBuildScrollInfoItemsForHoisting()
+   * in nsDisplayList.cpp.
+   */
   nsIFrame* current = aScrollFrame;
   while (current) {
-    if (UsesSVGEffects(current)) {
+    if (SVGIntegrationUtils::UsesSVGEffectsNotSupportedInCompositor(current)) {
       return true;
     }
     current = nsLayoutUtils::GetParentOrPlaceholderForCrossDoc(current);
@@ -1045,29 +1106,27 @@ void nsSliderFrame::StartAPZDrag(WidgetGUIEvent* aEvent) {
 
   bool isHorizontal = IsXULHorizontal();
 
-  mozilla::layers::FrameMetrics::ViewID scrollTargetId;
+  mozilla::layers::ScrollableLayerGuid::ViewID scrollTargetId;
   bool hasID = nsLayoutUtils::FindIDFor(scrollableContent, &scrollTargetId);
   bool hasAPZView =
-      hasID && (scrollTargetId != layers::FrameMetrics::NULL_SCROLL_ID);
+      hasID && (scrollTargetId != layers::ScrollableLayerGuid::NULL_SCROLL_ID);
 
   if (!hasAPZView) {
     return;
   }
 
-  nsCOMPtr<nsIContent> scrollbar = GetContentOfBox(scrollbarBox);
+  if (!DisplayPortUtils::HasNonMinimalDisplayPort(scrollableContent)) {
+    return;
+  }
 
-  nsIPresShell* shell = PresShell();
+  mozilla::PresShell* presShell = PresShell();
   uint64_t inputblockId = InputAPZContext::GetInputBlockId();
-  uint32_t presShellId = shell->GetPresShellId();
+  uint32_t presShellId = presShell->GetPresShellId();
   AsyncDragMetrics dragMetrics(
       scrollTargetId, presShellId, inputblockId,
       NSAppUnitsToFloatPixels(mDragStart, float(AppUnitsPerCSSPixel())),
       isHorizontal ? ScrollDirection::eHorizontal : ScrollDirection::eVertical);
 
-  if (!nsLayoutUtils::HasDisplayPort(scrollableContent)) {
-    return;
-  }
-
   // It's important to set this before calling
   // nsIWidget::StartAsyncScrollbarDrag(), because in some configurations, that
   // can call AsyncScrollbarDragRejected() synchronously, which clears the flag
@@ -1079,15 +1138,26 @@ void nsSliderFrame::StartAPZDrag(WidgetGUIEvent* aEvent) {
   bool waitForRefresh = InputAPZContext::HavePendingLayerization();
   nsIWidget* widget = this->GetNearestWidget();
   if (waitForRefresh) {
-    waitForRefresh = shell->AddPostRefreshObserver(
-        new AsyncScrollbarDragStarter(shell, widget, dragMetrics));
+    waitForRefresh = false;
+    if (nsPresContext* presContext = presShell->GetPresContext()) {
+      presContext->RegisterManagedPostRefreshObserver(
+          new ManagedPostRefreshObserver(
+              presContext, [widget = RefPtr<nsIWidget>(widget),
+                            dragMetrics](bool aWasCanceled) {
+                if (!aWasCanceled) {
+                  widget->StartAsyncScrollbarDrag(dragMetrics);
+                }
+                return ManagedPostRefreshObserver::Unregister::Yes;
+              }));
+      waitForRefresh = true;
+    }
   }
   if (!waitForRefresh) {
     widget->StartAsyncScrollbarDrag(dragMetrics);
   }
 }
 
-nsresult nsSliderFrame::StartDrag(nsIDOMEvent* aEvent) {
+nsresult nsSliderFrame::StartDrag(Event* aEvent) {
 #ifdef DEBUG_SLIDER
   printf("Begin dragging\n");
 #endif
@@ -1141,9 +1211,8 @@ nsresult nsSliderFrame::StartDrag(nsIDOMEvent* aEvent) {
   }
 
 #ifdef MOZ_WIDGET_GTK
-  RefPtr<Element> thumb = thumbFrame->GetContent()->AsElement();
-  thumb->SetAttr(kNameSpaceID_None, nsGkAtoms::active,
-                 NS_LITERAL_STRING("true"), true);
+  RefPtr<dom::Element> thumb = thumbFrame->GetContent()->AsElement();
+  thumb->SetAttr(kNameSpaceID_None, nsGkAtoms::active, u"true"_ns, true);
 #endif
 
   if (isHorizontal)
@@ -1178,7 +1247,7 @@ nsresult nsSliderFrame::StopDrag() {
 #ifdef MOZ_WIDGET_GTK
   nsIFrame* thumbFrame = mFrames.FirstChild();
   if (thumbFrame) {
-    RefPtr<Element> thumb = thumbFrame->GetContent()->AsElement();
+    RefPtr<dom::Element> thumb = thumbFrame->GetContent()->AsElement();
     thumb->UnsetAttr(kNameSpaceID_None, nsGkAtoms::active, true);
   }
 #endif
@@ -1193,24 +1262,16 @@ nsresult nsSliderFrame::StopDrag() {
 void nsSliderFrame::DragThumb(bool aGrabMouseEvents) {
   mDragFinished = !aGrabMouseEvents;
 
-  // inform the parent <scale> that a drag is beginning or ending
-  nsIFrame* parent = GetParent();
-  if (parent) {
-    nsCOMPtr<nsISliderListener> sliderListener =
-        do_QueryInterface(parent->GetContent());
-    if (sliderListener) {
-      nsContentUtils::AddScriptRunner(
-          new nsDragStateChangedRunnable(sliderListener, aGrabMouseEvents));
-    }
+  if (aGrabMouseEvents) {
+    PresShell::SetCapturingContent(GetContent(),
+                                   CaptureFlags::IgnoreAllowedState);
+  } else {
+    PresShell::ReleaseCapturingContent();
   }
-
-  nsIPresShell::SetCapturingContent(
-      aGrabMouseEvents ? GetContent() : nullptr,
-      aGrabMouseEvents ? CAPTURE_IGNOREALLOWED : 0);
 }
 
-bool nsSliderFrame::isDraggingThumb() {
-  return (nsIPresShell::GetCapturingContent() == GetContent());
+bool nsSliderFrame::isDraggingThumb() const {
+  return PresShell::GetCapturingContent() == GetContent();
 }
 
 void nsSliderFrame::AddListener() {
@@ -1222,10 +1283,10 @@ void nsSliderFrame::AddListener() {
   if (!thumbFrame) {
     return;
   }
-  thumbFrame->GetContent()->AddSystemEventListener(
-      NS_LITERAL_STRING("mousedown"), mMediator, false, false);
-  thumbFrame->GetContent()->AddSystemEventListener(
-      NS_LITERAL_STRING("touchstart"), mMediator, false, false);
+  thumbFrame->GetContent()->AddSystemEventListener(u"mousedown"_ns, mMediator,
+                                                   false, false);
+  thumbFrame->GetContent()->AddSystemEventListener(u"touchstart"_ns, mMediator,
+                                                   false, false);
 }
 
 void nsSliderFrame::RemoveListener() {
@@ -1234,8 +1295,10 @@ void nsSliderFrame::RemoveListener() {
   nsIFrame* thumbFrame = mFrames.FirstChild();
   if (!thumbFrame) return;
 
-  thumbFrame->GetContent()->RemoveSystemEventListener(
-      NS_LITERAL_STRING("mousedown"), mMediator, false);
+  thumbFrame->GetContent()->RemoveSystemEventListener(u"mousedown"_ns,
+                                                      mMediator, false);
+  thumbFrame->GetContent()->RemoveSystemEventListener(u"touchstart"_ns,
+                                                      mMediator, false);
 }
 
 bool nsSliderFrame::ShouldScrollForEvent(WidgetGUIEvent* aEvent) {
@@ -1245,15 +1308,15 @@ bool nsSliderFrame::ShouldScrollForEvent(WidgetGUIEvent* aEvent) {
       return true;
     case eMouseDown:
     case eMouseUp: {
-      uint16_t button = aEvent->AsMouseEvent()->button;
+      uint16_t button = aEvent->AsMouseEvent()->mButton;
 #ifdef MOZ_WIDGET_GTK
-      return (button == WidgetMouseEvent::eLeftButton) ||
-             (button == WidgetMouseEvent::eRightButton && GetScrollToClick()) ||
-             (button == WidgetMouseEvent::eMiddleButton && gMiddlePref &&
+      return (button == MouseButton::ePrimary) ||
+             (button == MouseButton::eSecondary && GetScrollToClick()) ||
+             (button == MouseButton::eMiddle && gMiddlePref &&
               !GetScrollToClick());
 #else
-      return (button == WidgetMouseEvent::eLeftButton) ||
-             (button == WidgetMouseEvent::eMiddleButton && gMiddlePref);
+      return (button == MouseButton::ePrimary) ||
+             (button == MouseButton::eMiddle && gMiddlePref);
 #endif
     }
     default:
@@ -1283,7 +1346,7 @@ bool nsSliderFrame::ShouldScrollToClickForEvent(WidgetGUIEvent* aEvent) {
   }
 
   WidgetMouseEvent* mouseEvent = aEvent->AsMouseEvent();
-  if (mouseEvent->button == WidgetMouseEvent::eLeftButton) {
+  if (mouseEvent->mButton == MouseButton::ePrimary) {
 #ifdef XP_MACOSX
     bool invertPref = mouseEvent->IsAlt();
 #else
@@ -1293,7 +1356,7 @@ bool nsSliderFrame::ShouldScrollToClickForEvent(WidgetGUIEvent* aEvent) {
   }
 
 #ifdef MOZ_WIDGET_GTK
-  if (mouseEvent->button == WidgetMouseEvent::eRightButton) {
+  if (mouseEvent->mButton == MouseButton::eSecondary) {
     return !GetScrollToClick();
   }
 #endif
@@ -1425,7 +1488,7 @@ nsSize nsSliderFrame::GetXULMinSize(nsBoxLayoutState& aState) {
   EnsureOrient();
 
   // our min size is just our borders and padding
-  return nsBox::GetXULMinSize(aState);
+  return nsIFrame::GetUncachedXULMinSize(aState);
 }
 
 nsSize nsSliderFrame::GetXULMaxSize(nsBoxLayoutState& aState) {
@@ -1436,8 +1499,7 @@ nsSize nsSliderFrame::GetXULMaxSize(nsBoxLayoutState& aState) {
 void nsSliderFrame::EnsureOrient() {
   nsIFrame* scrollbarBox = GetScrollbar();
 
-  bool isHorizontal =
-      (scrollbarBox->GetStateBits() & NS_STATE_IS_HORIZONTAL) != 0;
+  bool isHorizontal = scrollbarBox->HasAnyStateBits(NS_STATE_IS_HORIZONTAL);
   if (isHorizontal)
     AddStateBits(NS_STATE_IS_HORIZONTAL);
   else
@@ -1505,6 +1567,10 @@ float nsSliderFrame::GetThumbRatio() const {
   return mRatio / mozilla::AppUnitsPerCSSPixel();
 }
 
+void nsSliderFrame::AsyncScrollbarDragInitiated(uint64_t aDragBlockId) {
+  mAPZDragInitiated = Some(aDragBlockId);
+}
+
 void nsSliderFrame::AsyncScrollbarDragRejected() {
   mScrollingWithAPZ = false;
   // Only suppress the displayport if we're still dragging the thumb.
@@ -1516,25 +1582,24 @@ void nsSliderFrame::AsyncScrollbarDragRejected() {
 
 void nsSliderFrame::SuppressDisplayport() {
   if (!mSuppressionActive) {
-    MOZ_ASSERT(PresShell());
-    APZCCallbackHelper::SuppressDisplayport(true, PresShell());
+    PresShell()->SuppressDisplayport(true);
     mSuppressionActive = true;
   }
 }
 
 void nsSliderFrame::UnsuppressDisplayport() {
   if (mSuppressionActive) {
-    MOZ_ASSERT(PresShell());
-    APZCCallbackHelper::SuppressDisplayport(false, PresShell());
+    PresShell()->SuppressDisplayport(false);
     mSuppressionActive = false;
   }
 }
 
 bool nsSliderFrame::OnlySystemGroupDispatch(EventMessage aMessage) const {
-  // If we are in a native anonymous subtree, do not dispatch mouse-move events
-  // targeted at this slider frame to web content. This matches the behaviour
-  // of other browsers.
-  return aMessage == eMouseMove && GetContent()->IsInNativeAnonymousSubtree();
+  // If we are in a native anonymous subtree, do not dispatch mouse-move or
+  // pointer-move events targeted at this slider frame to web content. This
+  // matches the behaviour of other browsers.
+  return (aMessage == eMouseMove || aMessage == ePointerMove) &&
+         isDraggingThumb() && GetContent()->IsInNativeAnonymousSubtree();
 }
 
 NS_IMPL_ISUPPORTS(nsSliderMediator, nsIDOMEventListener)
