@@ -12,6 +12,7 @@
 #define nsXULPopupManager_h__
 
 #include "mozilla/Logging.h"
+#include "nsHashtablesFwd.h"
 #include "nsIContent.h"
 #include "nsIRollupListener.h"
 #include "nsIDOMEventListener.h"
@@ -22,14 +23,19 @@
 #include "nsITimer.h"
 #include "nsIReflowCallback.h"
 #include "nsThreadUtils.h"
+#include "nsPresContext.h"
 #include "nsStyleConsts.h"
 #include "nsWidgetInitData.h"
 #include "mozilla/Attributes.h"
+#include "mozilla/widget/NativeMenu.h"
 #include "Units.h"
 
+// XXX Avoid including this here by moving function bodies to the cpp file.
+#include "mozilla/dom/Element.h"
+
 // X.h defines KeyPress
 #ifdef KeyPress
-#undef KeyPress
+#  undef KeyPress
 #endif
 
 /**
@@ -57,11 +63,55 @@ class nsPIDOMWindowOuter;
 class nsRefreshDriver;
 
 namespace mozilla {
+class PresShell;
 namespace dom {
+class Event;
 class KeyboardEvent;
 }  // namespace dom
 }  // namespace mozilla
 
+// XUL popups can be in several different states. When opening a popup, the
+// state changes as follows:
+//   ePopupClosed - initial state
+//   ePopupShowing - during the period when the popupshowing event fires
+//   ePopupOpening - between the popupshowing event and being visible. Creation
+//                   of the child frames, layout and reflow occurs in this
+//                   state. The popup is stored in the popup manager's list of
+//                   open popups during this state.
+//   ePopupVisible - layout is done and the popup's view and widget are made
+//                   visible. The popup is visible on screen but may be
+//                   transitioning. The popupshown event has not yet fired.
+//   ePopupShown - the popup has been shown and is fully ready. This state is
+//                 assigned just before the popupshown event fires.
+// When closing a popup:
+//   ePopupHidden - during the period when the popuphiding event fires and
+//                  the popup is removed.
+//   ePopupClosed - the popup's widget is made invisible.
+enum nsPopupState {
+  // state when a popup is not open
+  ePopupClosed,
+  // state from when a popup is requested to be shown to after the
+  // popupshowing event has been fired.
+  ePopupShowing,
+  // state while a popup is waiting to be laid out and positioned
+  ePopupPositioning,
+  // state while a popup is open but the widget is not yet visible
+  ePopupOpening,
+  // state while a popup is visible and waiting for the popupshown event
+  ePopupVisible,
+  // state while a popup is open and visible on screen
+  ePopupShown,
+  // state from when a popup is requested to be hidden to when it is closed.
+  ePopupHiding,
+  // state which indicates that the popup was hidden without firing the
+  // popuphiding or popuphidden events. It is used when executing a menu
+  // command because the menu needs to be hidden before the command event
+  // fires, yet the popuphiding and popuphidden events are fired after. This
+  // state can also occur when the popup is removed because the document is
+  // unloaded.
+  ePopupInvisible
+};
+
 // when a menu command is executed, the closemenu attribute may be used
 // to define how the menu should be closed up
 enum CloseMenuMode {
@@ -118,7 +168,8 @@ enum nsIgnoreKeys {
 #define NS_DIRECTION_IS_BLOCK_TO_EDGE(dir) \
   (dir == eNavigationDirection_First || dir == eNavigationDirection_Last)
 
-static_assert(NS_STYLE_DIRECTION_LTR == 0 && NS_STYLE_DIRECTION_RTL == 1,
+static_assert(static_cast<uint8_t>(mozilla::StyleDirection::Ltr) == 0 &&
+                  static_cast<uint8_t>(mozilla::StyleDirection::Rtl) == 1,
               "Left to Right should be 0 and Right to Left should be 1");
 
 /**
@@ -128,10 +179,10 @@ static_assert(NS_STYLE_DIRECTION_LTR == 0 && NS_STYLE_DIRECTION_RTL == 1,
  */
 extern const nsNavigationDirection DirectionFromKeyCodeTable[2][6];
 
-#define NS_DIRECTION_FROM_KEY_CODE(frame, keycode)                 \
-  (DirectionFromKeyCodeTable[frame->StyleVisibility()->mDirection] \
-                            [keycode -                             \
-                             mozilla::dom::KeyboardEventBinding::DOM_VK_END])
+#define NS_DIRECTION_FROM_KEY_CODE(frame, keycode) \
+  (DirectionFromKeyCodeTable[static_cast<uint8_t>( \
+      (frame)->StyleVisibility()->mDirection)][(   \
+      keycode)-mozilla::dom::KeyboardEvent_Binding::DOM_VK_END])
 
 // nsMenuChainItem holds info about an open popup. Items are stored in a
 // doubly linked list. Note that the linked list is stored beginning from
@@ -171,7 +222,7 @@ class nsMenuChainItem {
     MOZ_COUNT_CTOR(nsMenuChainItem);
   }
 
-  ~nsMenuChainItem() { MOZ_COUNT_DTOR(nsMenuChainItem); }
+  MOZ_COUNTED_DTOR(nsMenuChainItem)
 
   nsIContent* Content();
   nsMenuPopupFrame* Frame() { return mFrame; }
@@ -201,27 +252,6 @@ class nsMenuChainItem {
   void Detach(nsMenuChainItem** aRoot);
 };
 
-// this class is used for dispatching popupshowing events asynchronously.
-class nsXULPopupShowingEvent : public mozilla::Runnable {
- public:
-  nsXULPopupShowingEvent(nsIContent* aPopup, bool aIsContextMenu,
-                         bool aSelectFirstItem)
-      : mozilla::Runnable("nsXULPopupShowingEvent"),
-        mPopup(aPopup),
-        mIsContextMenu(aIsContextMenu),
-        mSelectFirstItem(aSelectFirstItem) {
-    NS_ASSERTION(aPopup,
-                 "null popup supplied to nsXULPopupShowingEvent constructor");
-  }
-
-  NS_IMETHOD Run() override;
-
- private:
-  nsCOMPtr<nsIContent> mPopup;
-  bool mIsContextMenu;
-  bool mSelectFirstItem;
-};
-
 // this class is used for dispatching popuphiding events asynchronously.
 class nsXULPopupHidingEvent : public mozilla::Runnable {
  public:
@@ -254,42 +284,33 @@ class nsXULPopupHidingEvent : public mozilla::Runnable {
 // this class is used for dispatching popuppositioned events asynchronously.
 class nsXULPopupPositionedEvent : public mozilla::Runnable {
  public:
-  explicit nsXULPopupPositionedEvent(nsIContent* aPopup, bool aIsContextMenu,
-                                     bool aSelectFirstItem)
-      : mozilla::Runnable("nsXULPopupPositionedEvent"),
-        mPopup(aPopup),
-        mIsContextMenu(aIsContextMenu),
-        mSelectFirstItem(aSelectFirstItem) {
-    NS_ASSERTION(aPopup,
-                 "null popup supplied to nsXULPopupShowingEvent constructor");
+  explicit nsXULPopupPositionedEvent(nsIContent* aPopup)
+      : mozilla::Runnable("nsXULPopupPositionedEvent"), mPopup(aPopup) {
+    NS_ASSERTION(
+        aPopup, "null popup supplied to nsXULPopupPositionedEvent constructor");
   }
 
   NS_IMETHOD Run() override;
 
   // Asynchronously dispatch a popuppositioned event at aPopup if this is a
   // panel that should receieve such events. Return true if the event was sent.
-  static bool DispatchIfNeeded(nsIContent* aPopup, bool aIsContextMenu,
-                               bool aSelectFirstItem);
+  static bool DispatchIfNeeded(nsIContent* aPopup);
 
  private:
   nsCOMPtr<nsIContent> mPopup;
-  bool mIsContextMenu;
-  bool mSelectFirstItem;
 };
 
 // this class is used for dispatching menu command events asynchronously.
 class nsXULMenuCommandEvent : public mozilla::Runnable {
  public:
   nsXULMenuCommandEvent(mozilla::dom::Element* aMenu, bool aIsTrusted,
-                        bool aShift, bool aControl, bool aAlt, bool aMeta,
-                        bool aUserInput, bool aFlipChecked)
+                        mozilla::Modifiers aModifiers, bool aUserInput,
+                        bool aFlipChecked, int16_t aButton)
       : mozilla::Runnable("nsXULMenuCommandEvent"),
         mMenu(aMenu),
+        mModifiers(aModifiers),
+        mButton(aButton),
         mIsTrusted(aIsTrusted),
-        mShift(aShift),
-        mControl(aControl),
-        mAlt(aAlt),
-        mMeta(aMeta),
         mUserInput(aUserInput),
         mFlipChecked(aFlipChecked),
         mCloseMenuMode(CloseMenuMode_Auto) {
@@ -297,7 +318,7 @@ class nsXULMenuCommandEvent : public mozilla::Runnable {
                  "null menu supplied to nsXULMenuCommandEvent constructor");
   }
 
-  NS_IMETHOD Run() override;
+  MOZ_CAN_RUN_SCRIPT_BOUNDARY NS_IMETHOD Run() override;
 
   void SetCloseMenuMode(CloseMenuMode aCloseMenuMode) {
     mCloseMenuMode = aCloseMenuMode;
@@ -305,11 +326,10 @@ class nsXULMenuCommandEvent : public mozilla::Runnable {
 
  private:
   RefPtr<mozilla::dom::Element> mMenu;
+
+  mozilla::Modifiers mModifiers;
+  int16_t mButton;
   bool mIsTrusted;
-  bool mShift;
-  bool mControl;
-  bool mAlt;
-  bool mMeta;
   bool mUserInput;
   bool mFlipChecked;
   CloseMenuMode mCloseMenuMode;
@@ -317,9 +337,9 @@ class nsXULMenuCommandEvent : public mozilla::Runnable {
 
 class nsXULPopupManager final : public nsIDOMEventListener,
                                 public nsIRollupListener,
-                                public nsIObserver {
+                                public nsIObserver,
+                                public mozilla::widget::NativeMenu::Observer {
  public:
-  friend class nsXULPopupShowingEvent;
   friend class nsXULPopupHidingEvent;
   friend class nsXULPopupPositionedEvent;
   friend class nsXULMenuCommandEvent;
@@ -330,7 +350,9 @@ class nsXULPopupManager final : public nsIDOMEventListener,
   NS_DECL_NSIDOMEVENTLISTENER
 
   // nsIRollupListener
-  virtual bool Rollup(uint32_t aCount, bool aFlush, const nsIntPoint* pos,
+  MOZ_CAN_RUN_SCRIPT_BOUNDARY
+  virtual bool Rollup(uint32_t aCount, bool aFlush,
+                      const mozilla::LayoutDeviceIntPoint* pos,
                       nsIContent** aLastRolledUp) override;
   virtual bool ShouldRollupOnMouseWheelEvent() override;
   virtual bool ShouldConsumeOnMouseWheelEvent() override;
@@ -339,6 +361,16 @@ class nsXULPopupManager final : public nsIDOMEventListener,
       nsTArray<nsIWidget*>* aWidgetChain) override;
   virtual void NotifyGeometryChange() override {}
   virtual nsIWidget* GetRollupWidget() override;
+  virtual bool RollupNativeMenu() override;
+
+  // NativeMenu::Observer
+  void OnNativeMenuOpened() override;
+  void OnNativeMenuClosed() override;
+  void OnNativeSubMenuWillOpen(mozilla::dom::Element* aPopupElement) override;
+  void OnNativeSubMenuDidOpen(mozilla::dom::Element* aPopupElement) override;
+  void OnNativeSubMenuClosed(mozilla::dom::Element* aPopupElement) override;
+  void OnNativeMenuWillActivateItem(
+      mozilla::dom::Element* aMenuItemElement) override;
 
   static nsXULPopupManager* sInstance;
 
@@ -360,7 +392,7 @@ class nsXULPopupManager final : public nsIDOMEventListener,
   // This should be called when a window is moved or resized to adjust the
   // popups accordingly.
   void AdjustPopupsOnWindowChange(nsPIDOMWindowOuter* aWindow);
-  void AdjustPopupsOnWindowChange(nsIPresShell* aPresShell);
+  void AdjustPopupsOnWindowChange(mozilla::PresShell* aPresShell);
 
   // given a menu frame, find the prevous or next menu frame. If aPopup is
   // true then navigate a menupopup, from one item on the menu to the previous
@@ -408,10 +440,10 @@ class nsXULPopupManager final : public nsIDOMEventListener,
   // retrieve the node and offset of the last mouse event used to open a
   // context menu. This information is determined from the rangeParent and
   // the rangeOffset of the event supplied to ShowPopup or ShowPopupAtScreen.
-  // This is used by the implementation of XULDocument::GetPopupRangeParent
-  // and XULDocument::GetPopupRangeOffset.
-  nsINode* GetMouseLocationParent();
-  int32_t MouseLocationOffset();
+  // This is used by the implementation of Document::GetPopupRangeParent
+  // and Document::GetPopupRangeOffset.
+  nsIContent* GetMouseLocationParent() const { return mRangeParentContent; }
+  int32_t MouseLocationOffset() const { return mRangeOffset; }
 
   /**
    * Open a <menu> given its content node. If aSelectFirstItem is
@@ -424,7 +456,7 @@ class nsXULPopupManager final : public nsIDOMEventListener,
   /**
    * Open a popup, either anchored or unanchored. If aSelectFirstItem is
    * true, then the first item in the menu is selected. The arguments are
-   * similar to those for nsIPopupBoxObject::OpenPopup.
+   * similar to those for XULPopupElement::OpenPopup.
    *
    * aTriggerEvent should be the event that triggered the event. This is used
    * to determine the coordinates and trigger node for the popup. This may be
@@ -435,7 +467,7 @@ class nsXULPopupManager final : public nsIDOMEventListener,
   void ShowPopup(nsIContent* aPopup, nsIContent* aAnchorContent,
                  const nsAString& aPosition, int32_t aXPos, int32_t aYPos,
                  bool aIsContextMenu, bool aAttributesOverride,
-                 bool aSelectFirstItem, nsIDOMEvent* aTriggerEvent);
+                 bool aSelectFirstItem, mozilla::dom::Event* aTriggerEvent);
 
   /**
    * Open a popup at a specific screen position specified by aXPos and aYPos,
@@ -448,7 +480,8 @@ class nsXULPopupManager final : public nsIDOMEventListener,
    * cursor.
    */
   void ShowPopupAtScreen(nsIContent* aPopup, int32_t aXPos, int32_t aYPos,
-                         bool aIsContextMenu, nsIDOMEvent* aTriggerEvent);
+                         bool aIsContextMenu,
+                         mozilla::dom::Event* aTriggerEvent);
 
   /* Open a popup anchored at a screen rectangle specified by aRect.
    * The remaining arguments are similar to ShowPopup.
@@ -456,7 +489,19 @@ class nsXULPopupManager final : public nsIDOMEventListener,
   void ShowPopupAtScreenRect(nsIContent* aPopup, const nsAString& aPosition,
                              const nsIntRect& aRect, bool aIsContextMenu,
                              bool aAttributesOverride,
-                             nsIDOMEvent* aTriggerEvent);
+                             mozilla::dom::Event* aTriggerEvent);
+
+  /**
+   * Open a popup as a native menu, at a specific screen position specified by
+   * aXPos and aYPos, measured in CSS pixels.
+   *
+   * This fires the popupshowing event synchronously.
+   *
+   * Returns whether native menus are supported for aPopup on this platform.
+   */
+  bool ShowPopupAsNativeMenu(nsIContent* aPopup, int32_t aXPos, int32_t aYPos,
+                             bool aIsContextMenu,
+                             mozilla::dom::Event* aTriggerEvent);
 
   /**
    * Open a tooltip at a specific screen position specified by aXPos and aYPos,
@@ -467,17 +512,6 @@ class nsXULPopupManager final : public nsIDOMEventListener,
   void ShowTooltipAtScreen(nsIContent* aPopup, nsIContent* aTriggerContent,
                            int32_t aXPos, int32_t aYPos);
 
-  /**
-   * This method is provided only for compatibility with an older popup API.
-   * New code should not call this function and should call ShowPopup instead.
-   *
-   * This fires the popupshowing event synchronously.
-   */
-  void ShowPopupWithAnchorAlign(nsIContent* aPopup, nsIContent* aAnchorContent,
-                                nsAString& aAnchor, nsAString& aAlign,
-                                int32_t aXPos, int32_t aYPos,
-                                bool aIsContextMenu);
-
   /*
    * Hide a popup aPopup. If the popup is in a <menu>, then also inform the
    * menu that the popup is being hidden.
@@ -485,8 +519,8 @@ class nsXULPopupManager final : public nsIDOMEventListener,
    * aHideChain - true if the entire chain of menus should be closed. If false,
    *              only this popup is closed.
    * aDeselectMenu - true if the parent <menu> of the popup should be
-   * deselected. This will be false when the menu is closed by pressing the
-   *                 Escape key.
+   *                 deselected. This will be false when the menu is closed by
+   *                 pressing the Escape key.
    * aAsynchronous - true if the first popuphiding event should be sent
    *                 asynchrously. This should be true if HidePopup is called
    *                 from a frame.
@@ -498,6 +532,11 @@ class nsXULPopupManager final : public nsIDOMEventListener,
                  bool aAsynchronous, bool aIsCancel,
                  nsIContent* aLastPopup = nullptr);
 
+  /*
+   * Hide the popup of a <menu>.
+   */
+  void HideMenu(nsIContent* aMenu);
+
   /**
    * Hide a popup after a short delay. This is used when rolling over menu
    * items. This timer is stored in mCloseTimer. The timer may be cancelled and
@@ -541,6 +580,14 @@ class nsXULPopupManager final : public nsIDOMEventListener,
    */
   void ExecuteMenu(nsIContent* aMenu, nsXULMenuCommandEvent* aEvent);
 
+  /**
+   * If a native menu is open, and aItem is an item in the menu's subtree,
+   * execute the item with the help of the native menu and close the menu.
+   * Returns true if a native menu was open.
+   */
+  bool ActivateNativeMenuItem(nsIContent* aItem, mozilla::Modifiers aModifiers,
+                              int16_t aButton, mozilla::ErrorResult& aRv);
+
   /**
    * Return true if the popup for the supplied content node is open.
    */
@@ -569,11 +616,13 @@ class nsXULPopupManager final : public nsIDOMEventListener,
    * aDocument. aDocument must be non-null and be a document contained within
    * the same window hierarchy as the popup to retrieve.
    */
-  already_AddRefed<nsINode> GetLastTriggerPopupNode(nsIDocument* aDocument) {
+  already_AddRefed<nsINode> GetLastTriggerPopupNode(
+      mozilla::dom::Document* aDocument) {
     return GetLastTriggerNode(aDocument, false);
   }
 
-  already_AddRefed<nsINode> GetLastTriggerTooltipNode(nsIDocument* aDocument) {
+  already_AddRefed<nsINode> GetLastTriggerTooltipNode(
+      mozilla::dom::Document* aDocument) {
     return GetLastTriggerNode(aDocument, true);
   }
 
@@ -669,6 +718,8 @@ class nsXULPopupManager final : public nsIDOMEventListener,
   // Sets mIgnoreKeys of the Top Visible Menu Item
   nsresult UpdateIgnoreKeys(bool aIgnoreKeys);
 
+  nsPopupState GetPopupState(mozilla::dom::Element* aPopupElement);
+
   nsresult KeyUp(mozilla::dom::KeyboardEvent* aKeyEvent);
   nsresult KeyDown(mozilla::dom::KeyboardEvent* aKeyEvent);
   nsresult KeyPress(mozilla::dom::KeyboardEvent* aKeyEvent);
@@ -678,6 +729,7 @@ class nsXULPopupManager final : public nsIDOMEventListener,
   ~nsXULPopupManager();
 
   // get the nsMenuPopupFrame, if any, for the given content node
+  MOZ_CAN_RUN_SCRIPT_BOUNDARY
   nsMenuPopupFrame* GetPopupFrameForContent(nsIContent* aContent,
                                             bool aShouldFlush);
 
@@ -688,9 +740,15 @@ class nsXULPopupManager final : public nsIDOMEventListener,
   // cause style changes and frame destruction.
   void HidePopupsInList(const nsTArray<nsMenuPopupFrame*>& aFrames);
 
+  // Hide, but don't close, visible menus. Called before executing a menu item.
+  // The caller promises to close the menus properly (with a call to HidePopup)
+  // once the item has been executed.
+  void HideOpenMenusBeforeExecutingMenu(CloseMenuMode aMode);
+
   // set the event that was used to trigger the popup, or null to clear the
   // event details. aTriggerContent will be set to the target of the event.
-  void InitTriggerEvent(nsIDOMEvent* aEvent, nsIContent* aPopup,
+  MOZ_CAN_RUN_SCRIPT_BOUNDARY
+  void InitTriggerEvent(mozilla::dom::Event* aEvent, nsIContent* aPopup,
                         nsIContent** aTriggerContent);
 
   // callbacks for ShowPopup and HidePopup as events may be done asynchronously
@@ -701,7 +759,8 @@ class nsXULPopupManager final : public nsIDOMEventListener,
                          nsPopupType aPopupType, bool aDeselectMenu);
 
   /**
-   * Fire a popupshowing event on the popup and then open the popup.
+   * Trigger frame construction and reflow in the popup, fire a popupshowing
+   * event on the popup and then open the popup.
    *
    * aPopup - the popup to open
    * aIsContextMenu - true for context menus
@@ -710,8 +769,9 @@ class nsXULPopupManager final : public nsIDOMEventListener,
    *                 This is currently used to propagate the
    *                 inputSource attribute. May be null.
    */
-  void FirePopupShowingEvent(nsIContent* aPopup, bool aIsContextMenu,
-                             bool aSelectFirstItem, nsIDOMEvent* aTriggerEvent);
+  void BeginShowingPopup(nsIContent* aPopup, bool aIsContextMenu,
+                         bool aSelectFirstItem,
+                         mozilla::dom::Event* aTriggerEvent);
 
   /**
    * Fire a popuphiding event and then hide the popup. This will be called
@@ -733,6 +793,7 @@ class nsXULPopupManager final : public nsIDOMEventListener,
    * aDeselectMenu - true to unhighlight the menu when hiding it
    * aIsCancel - true if this popup is hiding due to being cancelled.
    */
+  MOZ_CAN_RUN_SCRIPT_BOUNDARY
   void FirePopupHidingEvent(nsIContent* aPopup, nsIContent* aNextPopup,
                             nsIContent* aLastPopup, nsPresContext* aPresContext,
                             nsPopupType aPopupType, bool aDeselectMenu,
@@ -759,8 +820,15 @@ class nsXULPopupManager final : public nsIDOMEventListener,
                                        nsNavigationDirection aDir);
 
  protected:
-  already_AddRefed<nsINode> GetLastTriggerNode(nsIDocument* aDocument,
-                                               bool aIsTooltip);
+  already_AddRefed<nsINode> GetLastTriggerNode(
+      mozilla::dom::Document* aDocument, bool aIsTooltip);
+
+  /**
+   * Fire a popupshowing event for aPopup.
+   */
+  nsEventStatus FirePopupShowingEvent(nsIContent* aPopup,
+                                      nsPresContext* aPresContext,
+                                      mozilla::dom::Event* aTriggerEvent);
 
   /**
    * Set mouse capturing for the current popup. This traps mouse clicks that
@@ -785,7 +853,8 @@ class nsXULPopupManager final : public nsIDOMEventListener,
   /*
    * Returns true if the docshell for aDoc is aExpected or a child of aExpected.
    */
-  bool IsChildOfDocShell(nsIDocument* aDoc, nsIDocShellTreeItem* aExpected);
+  bool IsChildOfDocShell(mozilla::dom::Document* aDoc,
+                         nsIDocShellTreeItem* aExpected);
 
   // the document the key event listener is attached to
   nsCOMPtr<mozilla::dom::EventTarget> mKeyListener;
@@ -794,7 +863,7 @@ class nsXULPopupManager final : public nsIDOMEventListener,
   nsCOMPtr<nsIWidget> mWidget;
 
   // range parent and offset set in SetTriggerEvent
-  nsCOMPtr<nsINode> mRangeParent;
+  nsCOMPtr<nsIContent> mRangeParentContent;
   int32_t mRangeOffset;
   // Device pixels relative to the showing popup's presshell's
   // root prescontext's root frame.
@@ -819,8 +888,24 @@ class nsXULPopupManager final : public nsIDOMEventListener,
   // popupshowing event
   nsCOMPtr<nsIContent> mOpeningPopup;
 
-  // If true, all popups won't hide automatically on blur
-  static bool sDevtoolsDisableAutoHide;
+  // If a popup is displayed as a native menu, this is non-null while the
+  // native menu is open.
+  // mNativeMenu has a strong reference to the menupopup nsIContent.
+  RefPtr<mozilla::widget::NativeMenu> mNativeMenu;
+
+  // If the currently open native menu activated an item, this is the item's
+  // close menu mode. Nothing() if mNativeMenu is null or if no item was
+  // activated.
+  mozilla::Maybe<CloseMenuMode> mNativeMenuActivatedItemCloseMenuMode;
+
+  // If a popup is displayed as a native menu, this map contains the popup state
+  // for any of its non-closed submenus. This state cannot be stored on the
+  // submenus' nsMenuPopupFrames, because we usually don't generate frames for
+  // the contents of native menus.
+  // If a submenu is not present in this map, it means it's closed.
+  // This map is empty if mNativeMenu is null.
+  nsTHashMap<RefPtr<mozilla::dom::Element>, nsPopupState>
+      mNativeMenuSubmenuStates;
 };
 
 #endif
