@@ -98,83 +98,91 @@ static LazyLogModule gIOServiceLog("nsIOService");
 // A general port blacklist.  Connections to these ports will not be allowed
 // unless the protocol overrides.
 //
-// TODO: I am sure that there are more ports to be added.
-//       This cut is based on the classic mozilla codebase
+// This list is to be kept in sync with "bad ports" as defined in the
+// WHATWG Fetch standard at <https://fetch.spec.whatwg.org/#port-blocking>
 
 int16_t gBadPortList[] = {
-    1,     // tcpmux
-    7,     // echo
-    9,     // discard
-    11,    // systat
-    13,    // daytime
-    15,    // netstat
-    17,    // qotd
-    19,    // chargen
-    20,    // ftp-data
-    21,    // ftp
-    22,    // ssh
-    23,    // telnet
-    25,    // smtp
-    37,    // time
-    42,    // name
-    43,    // nicname
-    53,    // domain
-    77,    // priv-rjs
-    79,    // finger
-    87,    // ttylink
-    95,    // supdup
-    101,   // hostriame
-    102,   // iso-tsap
-    103,   // gppitnp
-    104,   // acr-nema
-    109,   // pop2
-    110,   // pop3
-    111,   // sunrpc
-    113,   // auth
-    115,   // sftp
-    117,   // uucp-path
-    119,   // nntp
-    123,   // ntp
-    135,   // loc-srv / epmap
-    139,   // netbios
-    143,   // imap2
-    179,   // bgp
-    389,   // ldap
-    427,   // afp (alternate)
-    465,   // smtp (alternate)
-    512,   // print / exec
-    513,   // login
-    514,   // shell
-    515,   // printer
-    526,   // tempo
-    530,   // courier
-    531,   // chat
-    532,   // netnews
-    540,   // uucp
-    548,   // afp
-    554,   // rtsp
-    556,   // remotefs
-    563,   // nntp+ssl
-    587,   // smtp (outgoing)
-    601,   // syslog-conn
-    636,   // ldap+ssl
-    993,   // imap+ssl
-    995,   // pop3+ssl
-    1720,  // h323hostcall
-    1723,  // pptp
-    2049,  // nfs
-    3659,  // apple-sasl
-    4045,  // lockd
-    5060,  // sip
-    5061,  // sips
-    6000,  // x11
-    6665,  // irc (alternate)
-    6666,  // irc (alternate)
-    6667,  // irc (default)
-    6668,  // irc (alternate)
-    6669,  // irc (alternate)
-    6697,  // irc+tls
-    0,     // Sentinel value: This MUST be zero
+    1,      // tcpmux
+    7,      // echo
+    9,      // discard
+    11,     // systat
+    13,     // daytime
+    15,     // netstat
+    17,     // qotd
+    19,     // chargen
+    20,     // ftp-data
+    21,     // ftp
+    22,     // ssh
+    23,     // telnet
+    25,     // smtp
+    37,     // time
+    42,     // name
+    43,     // nicname
+    53,     // domain
+    69,     // tftp
+    77,     // priv-rjs
+    79,     // finger
+    87,     // ttylink
+    95,     // supdup
+    101,    // hostriame
+    102,    // iso-tsap
+    103,    // gppitnp
+    104,    // acr-nema
+    109,    // pop2
+    110,    // pop3
+    111,    // sunrpc
+    113,    // auth
+    115,    // sftp
+    117,    // uucp-path
+    119,    // nntp
+    123,    // ntp
+    135,    // loc-srv / epmap
+    137,    // netbios
+    139,    // netbios
+    143,    // imap2
+    161,    // snmp
+    179,    // bgp
+    389,    // ldap
+    427,    // afp (alternate)
+    465,    // smtp (alternate)
+    512,    // print / exec
+    513,    // login
+    514,    // shell
+    515,    // printer
+    526,    // tempo
+    530,    // courier
+    531,    // chat
+    532,    // netnews
+    540,    // uucp
+    548,    // afp
+    554,    // rtsp
+    556,    // remotefs
+    563,    // nntp+ssl
+    587,    // smtp (outgoing)
+    601,    // syslog-conn
+    636,    // ldap+ssl
+    989,    // ftps-data
+    990,    // ftps
+    993,    // imap+ssl
+    995,    // pop3+ssl
+    1719,   // h323gatestat
+    1720,   // h323hostcall
+    1723,   // pptp
+    2049,   // nfs
+    3659,   // apple-sasl
+    4045,   // lockd
+    5060,   // sip
+    5061,   // sips
+    6000,   // x11
+    6566,   // sane-port
+    6665,   // irc (alternate)
+    6666,   // irc (alternate)
+    6667,   // irc (default)
+    6668,   // irc (alternate)
+    6669,   // irc (alternate)
+    6697,   // irc+tls
+    10080,  // amanda
+    0,      // Sentinel value: This MUST be zero
 };
 
 static const char kProfileChangeNetTeardownTopic[] =
@@ -190,26 +198,9 @@ uint32_t nsIOService::gDefaultSegmentCount = 24;
 ////////////////////////////////////////////////////////////////////////////////
 
 nsIOService::nsIOService()
-    : mOffline(true),
-      mOfflineForProfileChange(false),
-      mManageLinkStatus(false),
-      mConnectivity(true),
-      mSettingOffline(false),
-      mSetOfflineValue(false),
-      mSocketProcessLaunchComplete(false),
-      mShutdown(false),
-      mHttpHandlerAlreadyShutingDown(false),
-      mNetworkLinkServiceInitialized(false),
-      mChannelEventSinks(NS_CHANNEL_EVENT_SINK_CATEGORY),
-      mMutex("nsIOService::mMutex"),
-      mTotalRequests(0),
-      mCacheWon(0),
-      mNetWon(0),
-      mLastOfflineStateChange(PR_IntervalNow()),
+    : mLastOfflineStateChange(PR_IntervalNow()),
       mLastConnectivityChange(PR_IntervalNow()),
-      mLastNetworkLinkChange(PR_IntervalNow()),
-      mNetTearingDownStarted(0),
-      mSocketProcess(nullptr) {}
+      mLastNetworkLinkChange(PR_IntervalNow()) {}
 
 static const char* gCallbackPrefs[] = {
     PORT_PREF_PREFIX,
@@ -226,8 +217,10 @@ static const char* gCallbackPrefsForSocketProcess[] = {
     "network.ssl_tokens_cache_enabled",
     "network.send_ODA_to_content_directly",
     "network.trr.",
+    "doh-rollout.",
     "network.dns.disableIPv6",
     "network.dns.skipTRR-when-parental-control-enabled",
+    "network.offline-mirrors-connectivity",
     nullptr,
 };
 
@@ -250,7 +243,6 @@ static const char* gCallbackSecurityPrefs[] = {
     "security.ssl.enable_ocsp_stapling",
     "security.ssl.enable_ocsp_must_staple",
     "security.pki.certificate_transparency.mode",
-    "security.cert_pinning.enforcement_level",
     "security.pki.name_matching_mode",
     nullptr,
 };
@@ -267,26 +259,37 @@ nsresult nsIOService::Init() {
   InitializeCaptivePortalService();
 
   // setup our bad port list stuff
-  for (int i = 0; gBadPortList[i]; i++)
+  for (int i = 0; gBadPortList[i]; i++) {
     mRestrictedPortList.AppendElement(gBadPortList[i]);
+  }
 
   // Further modifications to the port list come from prefs
   Preferences::RegisterPrefixCallbacks(nsIOService::PrefsChanged,
                                        gCallbackPrefs, this);
   PrefsChanged();
 
+  mSocketProcessTopicBlackList.Insert(
+      nsLiteralCString(NS_XPCOM_WILL_SHUTDOWN_OBSERVER_ID));
+  mSocketProcessTopicBlackList.Insert(
+      nsLiteralCString(NS_XPCOM_SHUTDOWN_OBSERVER_ID));
+  mSocketProcessTopicBlackList.Insert("xpcom-shutdown-threads"_ns);
+  mSocketProcessTopicBlackList.Insert("profile-do-change"_ns);
+
   // Register for profile change notifications
-  nsCOMPtr<nsIObserverService> observerService = services::GetObserverService();
-  if (observerService) {
-    observerService->AddObserver(this, kProfileChangeNetTeardownTopic, true);
-    observerService->AddObserver(this, kProfileChangeNetRestoreTopic, true);
-    observerService->AddObserver(this, kProfileDoChange, true);
-    observerService->AddObserver(this, NS_XPCOM_SHUTDOWN_OBSERVER_ID, true);
-    observerService->AddObserver(this, NS_NETWORK_LINK_TOPIC, true);
-    observerService->AddObserver(this, NS_NETWORK_ID_CHANGED_TOPIC, true);
-    observerService->AddObserver(this, NS_WIDGET_WAKE_OBSERVER_TOPIC, true);
-  } else
-    NS_WARNING("failed to get observer service");
+  mObserverService = services::GetObserverService();
+  AddObserver(this, kProfileChangeNetTeardownTopic, true);
+  AddObserver(this, kProfileChangeNetRestoreTopic, true);
+  AddObserver(this, kProfileDoChange, true);
+  AddObserver(this, NS_XPCOM_SHUTDOWN_OBSERVER_ID, true);
+  AddObserver(this, NS_NETWORK_LINK_TOPIC, true);
+  AddObserver(this, NS_NETWORK_ID_CHANGED_TOPIC, true);
+  AddObserver(this, NS_WIDGET_WAKE_OBSERVER_TOPIC, true);
+
+  // Register observers for sending notifications to nsSocketTransportService
+  if (XRE_IsParentProcess()) {
+    AddObserver(this, "profile-initial-state", true);
+    AddObserver(this, NS_WIDGET_SLEEP_OBSERVER_TOPIC, true);
+  }
 
   if (IsSocketProcessChild()) {
     Preferences::RegisterCallbacks(nsIOService::OnTLSPrefChange,
@@ -303,6 +306,65 @@ nsresult nsIOService::Init() {
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsIOService::AddObserver(nsIObserver* aObserver, const char* aTopic,
+                         bool aOwnsWeak) {
+  if (!mObserverService) {
+    return NS_ERROR_FAILURE;
+  }
+
+  // Register for the origional observer.
+  nsresult rv = mObserverService->AddObserver(aObserver, aTopic, aOwnsWeak);
+  if (NS_FAILED(rv)) {
+    return rv;
+  }
+
+  if (!XRE_IsParentProcess()) {
+    return NS_OK;
+  }
+
+  if (!UseSocketProcess()) {
+    return NS_OK;
+  }
+
+  nsAutoCString topic(aTopic);
+  if (mSocketProcessTopicBlackList.Contains(topic)) {
+    return NS_ERROR_FAILURE;
+  }
+
+  // Avoid registering  duplicate topics.
+  if (mObserverTopicForSocketProcess.Contains(topic)) {
+    return NS_ERROR_FAILURE;
+  }
+
+  mObserverTopicForSocketProcess.Insert(topic);
+
+  // This happens when AddObserver() is called by nsIOService::Init(). We don't
+  // want to add nsIOService again.
+  if (SameCOMIdentity(aObserver, static_cast<nsIObserver*>(this))) {
+    return NS_OK;
+  }
+
+  return mObserverService->AddObserver(this, aTopic, true);
+}
+
+NS_IMETHODIMP
+nsIOService::RemoveObserver(nsIObserver* aObserver, const char* aTopic) {
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+NS_IMETHODIMP
+nsIOService::EnumerateObservers(const char* aTopic,
+                                nsISimpleEnumerator** anEnumerator) {
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+NS_IMETHODIMP nsIOService::NotifyObservers(nsISupports* aSubject,
+                                           const char* aTopic,
+                                           const char16_t* aSomeData) {
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
 nsIOService::~nsIOService() {
   if (gIOService) {
     MOZ_ASSERT(gIOService == this);
@@ -327,7 +389,6 @@ void nsIOService::OnTLSPrefChange(const char* aPref, void* aSelf) {
   } else if (pref.EqualsLiteral("security.ssl.enable_ocsp_stapling") ||
              pref.EqualsLiteral("security.ssl.enable_ocsp_must_staple") ||
              pref.EqualsLiteral("security.pki.certificate_transparency.mode") ||
-             pref.EqualsLiteral("security.cert_pinning.enforcement_level") ||
              pref.EqualsLiteral("security.pki.name_matching_mode")) {
     SetValidationOptionsCommon();
   }
@@ -448,6 +509,10 @@ nsresult nsIOService::LaunchSocketProcess() {
     return NS_OK;
   }
 
+  if (mShutdown) {
+    return NS_OK;
+  }
+
   if (mSocketProcess) {
     return NS_OK;
   }
@@ -480,6 +545,7 @@ nsresult nsIOService::LaunchSocketProcess() {
 }
 
 void nsIOService::DestroySocketProcess() {
+  LOG(("nsIOService::DestroySocketProcess"));
   MOZ_ASSERT(NS_IsMainThread());
 
   if (XRE_GetProcessType() != GeckoProcessType_Default || !mSocketProcess) {
@@ -514,6 +580,11 @@ bool nsIOService::UseSocketProcess(bool aCheckAgain) {
     return sUseSocketProcess;
   }
 
+  if (PR_GetEnv("MOZ_FORCE_USE_SOCKET_PROCESS")) {
+    sUseSocketProcess = true;
+    return sUseSocketProcess;
+  }
+
   if (StaticPrefs::network_process_enabled()) {
     sUseSocketProcess =
         StaticPrefs::network_http_network_access_on_socket_process_enabled();
@@ -560,8 +631,7 @@ void nsIOService::OnProcessLaunchComplete(SocketProcessHost* aHost,
   }
 
   if (!mPendingEvents.IsEmpty()) {
-    nsTArray<std::function<void()>> pendingEvents;
-    mPendingEvents.SwapElements(pendingEvents);
+    nsTArray<std::function<void()>> pendingEvents = std::move(mPendingEvents);
     for (auto& func : pendingEvents) {
       func();
     }
@@ -622,7 +692,8 @@ nsIOService::SocketProcessTelemetryPing() {
 }
 
 NS_IMPL_ISUPPORTS(nsIOService, nsIIOService, nsINetUtil, nsISpeculativeConnect,
-                  nsIObserver, nsIIOServiceInternal, nsISupportsWeakReference)
+                  nsIObserver, nsIIOServiceInternal, nsISupportsWeakReference,
+                  nsIObserverService)
 
 ////////////////////////////////////////////////////////////////////////////////
 
@@ -656,17 +727,10 @@ nsresult nsIOService::RecheckCaptivePortalIfLocalRedirect(nsIChannel* newChan) {
     return rv;
   }
 
-  PRNetAddr prAddr;
-  if (PR_StringToNetAddr(host.BeginReading(), &prAddr) != PR_SUCCESS) {
-    // The redirect wasn't to an IP literal, so there's probably no need
-    // to trigger the captive portal detection right now. It can wait.
-    return NS_OK;
-  }
-
-  NetAddr netAddr;
-  PRNetAddrToNetAddr(&prAddr, &netAddr);
-  if (IsIPAddrLocal(&netAddr)) {
-    // Redirects to local IP addresses are probably captive portals
+  NetAddr addr;
+  // If the redirect wasn't to an IP literal, so there's probably no need
+  // to trigger the captive portal detection right now. It can wait.
+  if (NS_SUCCEEDED(addr.InitFromString(host)) && addr.IsIPAddrLocal()) {
     RecheckCaptivePortal();
   }
 
@@ -773,23 +837,14 @@ nsresult nsIOService::GetCachedProtocolHandler(const char* scheme,
 }
 
 static bool UsesExternalProtocolHandler(const char* aScheme) {
-  if (NS_LITERAL_CSTRING("file").Equals(aScheme) ||
-      NS_LITERAL_CSTRING("chrome").Equals(aScheme) ||
-      NS_LITERAL_CSTRING("resource").Equals(aScheme)) {
+  if ("file"_ns.Equals(aScheme) || "chrome"_ns.Equals(aScheme) ||
+      "resource"_ns.Equals(aScheme)) {
     // Don't allow file:, chrome: or resource: URIs to be handled with
     // nsExternalProtocolHandler, since internally we rely on being able to
     // use and read from these URIs.
     return false;
   }
 
-  // When ftp protocol is disabled, return true if external protocol handler was
-  // not explicitly disabled by the prererence.
-  if (NS_LITERAL_CSTRING("ftp").Equals(aScheme) &&
-      !Preferences::GetBool("network.ftp.enabled", true) &&
-      Preferences::GetBool("network.protocol-handler.external.ftp", true)) {
-    return true;
-  }
-
   for (const auto& forcedExternalScheme : gForcedExternalSchemes) {
     if (!nsCRT::strcasecmp(forcedExternalScheme, aScheme)) {
       return true;
@@ -870,14 +925,9 @@ nsIOService::HostnameIsLocalIPAddress(nsIURI* aURI, bool* aResult) {
 
   *aResult = false;
 
-  PRNetAddr addr;
-  PRStatus result = PR_StringToNetAddr(host.get(), &addr);
-  if (result == PR_SUCCESS) {
-    NetAddr netAddr;
-    PRNetAddrToNetAddr(&addr, &netAddr);
-    if (IsIPAddrLocal(&netAddr)) {
-      *aResult = true;
-    }
+  NetAddr addr;
+  if (NS_SUCCEEDED(addr.InitFromString(host)) && addr.IsIPAddrLocal()) {
+    *aResult = true;
   }
 
   return NS_OK;
@@ -898,14 +948,9 @@ nsIOService::HostnameIsSharedIPAddress(nsIURI* aURI, bool* aResult) {
 
   *aResult = false;
 
-  PRNetAddr addr;
-  PRStatus result = PR_StringToNetAddr(host.get(), &addr);
-  if (result == PR_SUCCESS) {
-    NetAddr netAddr;
-    PRNetAddrToNetAddr(&addr, &netAddr);
-    if (IsIPAddrShared(&netAddr)) {
-      *aResult = true;
-    }
+  NetAddr addr;
+  if (NS_SUCCEEDED(addr.InitFromString(host)) && addr.IsIPAddrShared()) {
+    *aResult = true;
   }
 
   return NS_OK;
@@ -966,8 +1011,7 @@ already_AddRefed<nsIURI> nsIOService::CreateExposableURI(nsIURI* aURI) {
   nsAutoCString userPass;
   uri->GetUserPass(userPass);
   if (!userPass.IsEmpty()) {
-    DebugOnly<nsresult> rv =
-        NS_MutateURI(uri).SetUserPass(EmptyCString()).Finalize(uri);
+    DebugOnly<nsresult> rv = NS_MutateURI(uri).SetUserPass(""_ns).Finalize(uri);
     MOZ_ASSERT(NS_SUCCEEDED(rv) && uri, "Mutating URI should never fail");
   }
   return uri.forget();
@@ -987,7 +1031,7 @@ nsIOService::NewChannelFromURI(nsIURI* aURI, nsINode* aLoadingNode,
                                nsIPrincipal* aLoadingPrincipal,
                                nsIPrincipal* aTriggeringPrincipal,
                                uint32_t aSecurityFlags,
-                               uint32_t aContentPolicyType,
+                               nsContentPolicyType aContentPolicyType,
                                nsIChannel** result) {
   return NewChannelFromURIWithProxyFlags(aURI,
                                          nullptr,  // aProxyURI
@@ -1001,7 +1045,8 @@ nsresult nsIOService::NewChannelFromURIWithClientAndController(
     nsIPrincipal* aTriggeringPrincipal,
     const Maybe<ClientInfo>& aLoadingClientInfo,
     const Maybe<ServiceWorkerDescriptor>& aController, uint32_t aSecurityFlags,
-    uint32_t aContentPolicyType, uint32_t aSandboxFlags, nsIChannel** aResult) {
+    nsContentPolicyType aContentPolicyType, uint32_t aSandboxFlags,
+    nsIChannel** aResult) {
   return NewChannelFromURIWithProxyFlagsInternal(
       aURI,
       nullptr,  // aProxyURI
@@ -1025,7 +1070,8 @@ nsresult nsIOService::NewChannelFromURIWithProxyFlagsInternal(
     nsIPrincipal* aTriggeringPrincipal,
     const Maybe<ClientInfo>& aLoadingClientInfo,
     const Maybe<ServiceWorkerDescriptor>& aController, uint32_t aSecurityFlags,
-    uint32_t aContentPolicyType, uint32_t aSandboxFlags, nsIChannel** result) {
+    nsContentPolicyType aContentPolicyType, uint32_t aSandboxFlags,
+    nsIChannel** result) {
   nsCOMPtr<nsILoadInfo> loadInfo = new LoadInfo(
       aLoadingPrincipal, aTriggeringPrincipal, aLoadingNode, aSecurityFlags,
       aContentPolicyType, aLoadingClientInfo, aController, aSandboxFlags);
@@ -1109,7 +1155,7 @@ nsIOService::NewChannelFromURIWithProxyFlags(
     nsIURI* aURI, nsIURI* aProxyURI, uint32_t aProxyFlags,
     nsINode* aLoadingNode, nsIPrincipal* aLoadingPrincipal,
     nsIPrincipal* aTriggeringPrincipal, uint32_t aSecurityFlags,
-    uint32_t aContentPolicyType, nsIChannel** result) {
+    nsContentPolicyType aContentPolicyType, nsIChannel** result) {
   return NewChannelFromURIWithProxyFlagsInternal(
       aURI, aProxyURI, aProxyFlags, aLoadingNode, aLoadingPrincipal,
       aTriggeringPrincipal, Maybe<ClientInfo>(),
@@ -1122,7 +1168,8 @@ nsIOService::NewChannel(const nsACString& aSpec, const char* aCharset,
                         nsIURI* aBaseURI, nsINode* aLoadingNode,
                         nsIPrincipal* aLoadingPrincipal,
                         nsIPrincipal* aTriggeringPrincipal,
-                        uint32_t aSecurityFlags, uint32_t aContentPolicyType,
+                        uint32_t aSecurityFlags,
+                        nsContentPolicyType aContentPolicyType,
                         nsIChannel** result) {
   nsresult rv;
   nsCOMPtr<nsIURI> uri;
@@ -1167,8 +1214,9 @@ nsIOService::SetOffline(bool offline) {
   LOG(("nsIOService::SetOffline offline=%d\n", offline));
   // When someone wants to go online (!offline) after we got XPCOM shutdown
   // throw ERROR_NOT_AVAILABLE to prevent return to online state.
-  if ((mShutdown || mOfflineForProfileChange) && !offline)
+  if ((mShutdown || mOfflineForProfileChange) && !offline) {
     return NS_ERROR_NOT_AVAILABLE;
+  }
 
   // SetOffline() may re-enter while it's shutting down services.
   // If that happens, save the most recent value and it will be
@@ -1191,6 +1239,9 @@ nsIOService::SetOffline(bool offline) {
                                              NS_IPC_IOSERVICE_SET_OFFLINE_TOPIC,
                                              offline ? u"true" : u"false");
     }
+    if (SocketProcessReady()) {
+      Unused << mSocketProcess->GetActor()->SendSetOffline(offline);
+    }
   }
 
   nsIIOService* subject = static_cast<nsIIOService*>(this);
@@ -1201,18 +1252,20 @@ nsIOService::SetOffline(bool offline) {
       mOffline = true;  // indicate we're trying to shutdown
 
       // don't care if notifications fail
-      if (observerService)
+      if (observerService) {
         observerService->NotifyObservers(subject,
                                          NS_IOSERVICE_GOING_OFFLINE_TOPIC,
                                          u"" NS_IOSERVICE_OFFLINE);
+      }
 
       if (mSocketTransportService) mSocketTransportService->SetOffline(true);
 
       mLastOfflineStateChange = PR_IntervalNow();
-      if (observerService)
+      if (observerService) {
         observerService->NotifyObservers(subject,
                                          NS_IOSERVICE_OFFLINE_STATUS_TOPIC,
                                          u"" NS_IOSERVICE_OFFLINE);
+      }
     } else if (!offline && mOffline) {
       // go online
       InitializeSocketTransportService();
@@ -1292,6 +1345,9 @@ nsresult nsIOService::SetConnectivityInternal(bool aConnectivity) {
     observerService->NotifyObservers(nullptr,
                                      NS_IPC_IOSERVICE_SET_CONNECTIVITY_TOPIC,
                                      aConnectivity ? u"true" : u"false");
+    if (SocketProcessReady()) {
+      Unused << mSocketProcess->GetActor()->SendSetConnectivity(aConnectivity);
+    }
   }
 
   if (mOffline) {
@@ -1326,7 +1382,7 @@ nsIOService::AllowPort(int32_t inPort, const char* scheme, bool* _retval) {
     return NS_OK;
   }
 
-  if (port == 0) {
+  if (port <= 0 || port > std::numeric_limits<uint16_t>::max()) {
     *_retval = false;
     return NS_OK;
   }
@@ -1336,7 +1392,6 @@ nsIOService::AllowPort(int32_t inPort, const char* scheme, bool* _retval) {
     MutexAutoLock lock(mMutex);
     restrictedPortList.Assign(mRestrictedPortList);
   }
-
   // first check to see if the port is in our blacklist:
   int32_t badPortListCnt = restrictedPortList.Length();
   for (int i = 0; i < badPortListCnt; i++) {
@@ -1372,12 +1427,14 @@ void nsIOService::PrefsChanged(const char* pref, void* self) {
 
 void nsIOService::PrefsChanged(const char* pref) {
   // Look for extra ports to block
-  if (!pref || strcmp(pref, PORT_PREF("banned")) == 0)
+  if (!pref || strcmp(pref, PORT_PREF("banned")) == 0) {
     ParsePortList(PORT_PREF("banned"), false);
+  }
 
   // ...as well as previous blocks to remove.
-  if (!pref || strcmp(pref, PORT_PREF("banned.override")) == 0)
+  if (!pref || strcmp(pref, PORT_PREF("banned.override")) == 0) {
     ParsePortList(PORT_PREF("banned.override"), true);
+  }
 
   if (!pref || strcmp(pref, MANAGE_OFFLINE_STATUS_PREF) == 0) {
     bool manage;
@@ -1393,20 +1450,23 @@ void nsIOService::PrefsChanged(const char* pref) {
   if (!pref || strcmp(pref, NECKO_BUFFER_CACHE_COUNT_PREF) == 0) {
     int32_t count;
     if (NS_SUCCEEDED(
-            Preferences::GetInt(NECKO_BUFFER_CACHE_COUNT_PREF, &count)))
+            Preferences::GetInt(NECKO_BUFFER_CACHE_COUNT_PREF, &count))) {
       /* check for bogus values and default if we find such a value */
       if (count > 0) gDefaultSegmentCount = count;
+    }
   }
 
   if (!pref || strcmp(pref, NECKO_BUFFER_CACHE_SIZE_PREF) == 0) {
     int32_t size;
-    if (NS_SUCCEEDED(Preferences::GetInt(NECKO_BUFFER_CACHE_SIZE_PREF, &size)))
+    if (NS_SUCCEEDED(
+            Preferences::GetInt(NECKO_BUFFER_CACHE_SIZE_PREF, &size))) {
       /* check for bogus values and default if we find such a value
        * the upper limit here is arbitrary. having a 1mb segment size
        * is pretty crazy.  if you remove this, consider adding some
        * integer rollover test.
        */
       if (size > 0 && size < 1024 * 1024) gDefaultSegmentSize = size;
+    }
     NS_WARNING_ASSERTION(!(size & (size - 1)),
                          "network segment size is not a power of 2!");
   }
@@ -1447,21 +1507,24 @@ void nsIOService::ParsePortList(const char* pref, bool remove) {
         if ((portBegin < 65536) && (portEnd < 65536)) {
           int32_t curPort;
           if (remove) {
-            for (curPort = portBegin; curPort <= portEnd; curPort++)
+            for (curPort = portBegin; curPort <= portEnd; curPort++) {
               restrictedPortList.RemoveElement(curPort);
+            }
           } else {
-            for (curPort = portBegin; curPort <= portEnd; curPort++)
+            for (curPort = portBegin; curPort <= portEnd; curPort++) {
               restrictedPortList.AppendElement(curPort);
+            }
           }
         }
       } else {
         nsresult aErrorCode;
         int32_t port = portListArray[index].ToInteger(&aErrorCode);
         if (NS_SUCCEEDED(aErrorCode) && port < 65536) {
-          if (remove)
+          if (remove) {
             restrictedPortList.RemoveElement(port);
-          else
+          } else {
             restrictedPortList.AppendElement(port);
+          }
         }
       }
     }
@@ -1525,7 +1588,7 @@ nsIOService::Observe(nsISupports* subject, const char* topic,
       SetOffline(false);
     }
   } else if (!strcmp(topic, kProfileDoChange)) {
-    if (data && NS_LITERAL_STRING("startup").Equals(data)) {
+    if (data && u"startup"_ns.Equals(data)) {
       // Lazy initialization of network link service (see bug 620472)
       InitializeNetworkLinkService();
       // Set up the initilization flag regardless the actuall result.
@@ -1580,6 +1643,17 @@ nsIOService::Observe(nsISupports* subject, const char* topic,
     NS_DispatchToMainThread(wakeupNotifier);
   }
 
+  if (UseSocketProcess() &&
+      mObserverTopicForSocketProcess.Contains(nsDependentCString(topic))) {
+    nsCString topicStr(topic);
+    nsString dataStr(data);
+    auto sendObserver = [topicStr, dataStr]() {
+      Unused << gIOService->mSocketProcess->GetActor()->SendNotifyObserver(
+          topicStr, dataStr);
+    };
+    CallOrWaitForSocketProcess(sendObserver);
+  }
+
   return NS_OK;
 }
 
@@ -1740,8 +1814,9 @@ nsIOService::EscapeString(const nsACString& aString, uint32_t aEscapeType,
   nsAutoCString stringCopy(aString);
   nsCString result;
 
-  if (!NS_Escape(stringCopy, result, (nsEscapeMask)aEscapeType))
+  if (!NS_Escape(stringCopy, result, (nsEscapeMask)aEscapeType)) {
     return NS_ERROR_OUT_OF_MEMORY;
+  }
 
   aResult.Assign(result);
 
@@ -1882,13 +1957,13 @@ nsresult nsIOService::SpeculativeConnectInternal(
   // channel we create underneath - hence it's safe to use
   // the systemPrincipal as the loadingPrincipal for this channel.
   nsCOMPtr<nsIChannel> channel;
-  rv = NewChannelFromURI(aURI,
-                         nullptr,  // aLoadingNode,
-                         loadingPrincipal,
-                         nullptr,  // aTriggeringPrincipal,
-                         nsILoadInfo::SEC_ALLOW_CROSS_ORIGIN_DATA_IS_NULL,
-                         nsIContentPolicy::TYPE_SPECULATIVE,
-                         getter_AddRefs(channel));
+  rv = NewChannelFromURI(
+      aURI,
+      nullptr,  // aLoadingNode,
+      loadingPrincipal,
+      nullptr,  // aTriggeringPrincipal,
+      nsILoadInfo::SEC_ALLOW_CROSS_ORIGIN_SEC_CONTEXT_IS_NULL,
+      nsIContentPolicy::TYPE_SPECULATIVE, getter_AddRefs(channel));
   NS_ENSURE_SUCCESS(rv, rv);
 
   if (aAnonymous) {
