@@ -1,26 +1,23 @@
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
- * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
+ * file, You can obtain one at https://mozilla.org/MPL/2.0/. */
 
 //! Per-node data used in style calculation.
 
-use context::{SharedStyleContext, StackLimitChecker};
-use dom::TElement;
-use invalidation::element::invalidator::InvalidationResult;
-use invalidation::element::restyle_hints::RestyleHint;
+use crate::context::{SharedStyleContext, StackLimitChecker};
+use crate::dom::TElement;
+use crate::invalidation::element::invalidator::InvalidationResult;
+use crate::invalidation::element::restyle_hints::RestyleHint;
+use crate::properties::ComputedValues;
+use crate::selector_parser::{PseudoElement, RestyleDamage, EAGER_PSEUDO_COUNT};
+use crate::style_resolver::{PrimaryStyle, ResolvedElementStyles, ResolvedStyle};
 #[cfg(feature = "gecko")]
 use malloc_size_of::MallocSizeOfOps;
-use properties::ComputedValues;
-use rule_tree::StrongRuleNode;
-use selector_parser::{EAGER_PSEUDO_COUNT, PseudoElement, RestyleDamage};
 use selectors::NthIndexCache;
 use servo_arc::Arc;
-use shared_lock::StylesheetGuards;
-use smallvec::SmallVec;
 use std::fmt;
 use std::mem;
 use std::ops::{Deref, DerefMut};
-use style_resolver::{PrimaryStyle, ResolvedElementStyles, ResolvedStyle};
 
 bitflags! {
     /// Various flags stored on ElementData.
@@ -95,7 +92,12 @@ impl fmt::Debug for EagerPseudoArray {
         write!(f, "EagerPseudoArray {{ ")?;
         for i in 0..EAGER_PSEUDO_COUNT {
             if let Some(ref values) = self[i] {
-                write!(f, "{:?}: {:?}, ", PseudoElement::from_eager_index(i), &values.rules)?;
+                write!(
+                    f,
+                    "{:?}: {:?}, ",
+                    PseudoElement::from_eager_index(i),
+                    &values.rules
+                )?;
             }
         }
         write!(f, "}}")
@@ -132,7 +134,9 @@ impl EagerPseudoStyles {
     /// Returns a reference to the style for a given eager pseudo, if it exists.
     pub fn get(&self, pseudo: &PseudoElement) -> Option<&Arc<ComputedValues>> {
         debug_assert!(pseudo.is_eager());
-        self.0.as_ref().and_then(|p| p[pseudo.eager_index()].as_ref())
+        self.0
+            .as_ref()
+            .and_then(|p| p[pseudo.eager_index()].as_ref())
     }
 
     /// Sets the style for the eager pseudo.
@@ -171,6 +175,25 @@ impl ElementStyles {
         self.primary().get_box().clone_display().is_none()
     }
 
+    /// Whether this element uses viewport units.
+    pub fn uses_viewport_units(&self) -> bool {
+        use crate::computed_value_flags::ComputedValueFlags;
+
+        if self.primary().flags.intersects(ComputedValueFlags::USES_VIEWPORT_UNITS) {
+            return true;
+        }
+
+        for pseudo_style in self.pseudos.as_array() {
+            if let Some(ref pseudo_style) = pseudo_style {
+                if pseudo_style.flags.intersects(ComputedValueFlags::USES_VIEWPORT_UNITS) {
+                    return true;
+                }
+            }
+        }
+
+        false
+    }
+
     #[cfg(feature = "gecko")]
     fn size_of_excluding_cvs(&self, _ops: &mut MallocSizeOfOps) -> usize {
         // As the method name suggests, we don't measures the ComputedValues
@@ -188,8 +211,12 @@ impl ElementStyles {
 // substitute the rule node instead.
 impl fmt::Debug for ElementStyles {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        write!(f, "ElementStyles {{ primary: {:?}, pseudos: {:?} }}",
-               self.primary.as_ref().map(|x| &x.rules), self.pseudos)
+        write!(
+            f,
+            "ElementStyles {{ primary: {:?}, pseudos: {:?} }}",
+            self.primary.as_ref().map(|x| &x.rules),
+            self.pseudos
+        )
     }
 }
 
@@ -245,41 +272,27 @@ impl ElementData {
             return InvalidationResult::empty();
         }
 
-        use invalidation::element::invalidator::TreeStyleInvalidator;
-        use invalidation::element::state_and_attributes::StateAndAttrInvalidationProcessor;
+        use crate::invalidation::element::invalidator::TreeStyleInvalidator;
+        use crate::invalidation::element::state_and_attributes::StateAndAttrInvalidationProcessor;
 
-        debug!("invalidate_style_if_needed: {:?}, flags: {:?}, has_snapshot: {}, \
-                handled_snapshot: {}, pseudo: {:?}",
-                element,
-                shared_context.traversal_flags,
-                element.has_snapshot(),
-                element.handled_snapshot(),
-                element.implemented_pseudo_element());
+        debug!(
+            "invalidate_style_if_needed: {:?}, flags: {:?}, has_snapshot: {}, \
+             handled_snapshot: {}, pseudo: {:?}",
+            element,
+            shared_context.traversal_flags,
+            element.has_snapshot(),
+            element.handled_snapshot(),
+            element.implemented_pseudo_element()
+        );
 
         if !element.has_snapshot() || element.handled_snapshot() {
             return InvalidationResult::empty();
         }
 
-        let mut non_document_styles = SmallVec::<[_; 3]>::new();
-        let matches_doc_author_rules =
-            element.each_applicable_non_document_style_rule_data(|data, quirks_mode| {
-                non_document_styles.push((data, quirks_mode))
-            });
+        let mut processor =
+            StateAndAttrInvalidationProcessor::new(shared_context, element, self, nth_index_cache);
 
-        let mut processor = StateAndAttrInvalidationProcessor::new(
-            shared_context,
-            &non_document_styles,
-            matches_doc_author_rules,
-            element,
-            self,
-            nth_index_cache,
-        );
-
-        let invalidator = TreeStyleInvalidator::new(
-            element,
-            stack_limit_checker,
-            &mut processor,
-        );
+        let invalidator = TreeStyleInvalidator::new(element, stack_limit_checker, &mut processor);
 
         let result = invalidator.invalidate();
 
@@ -305,8 +318,9 @@ impl ElementData {
 
     /// Returns this element's primary style as a resolved style to use for sharing.
     pub fn share_primary_style(&self) -> PrimaryStyle {
-        let reused_via_rule_node =
-            self.flags.contains(ElementDataFlags::PRIMARY_STYLE_REUSED_VIA_RULE_NODE);
+        let reused_via_rule_node = self
+            .flags
+            .contains(ElementDataFlags::PRIMARY_STYLE_REUSED_VIA_RULE_NODE);
 
         PrimaryStyle {
             style: ResolvedStyle(self.styles.primary().clone()),
@@ -317,19 +331,18 @@ impl ElementData {
     /// Sets a new set of styles, returning the old ones.
     pub fn set_styles(&mut self, new_styles: ResolvedElementStyles) -> ElementStyles {
         if new_styles.primary.reused_via_rule_node {
-            self.flags.insert(ElementDataFlags::PRIMARY_STYLE_REUSED_VIA_RULE_NODE);
+            self.flags
+                .insert(ElementDataFlags::PRIMARY_STYLE_REUSED_VIA_RULE_NODE);
         } else {
-            self.flags.remove(ElementDataFlags::PRIMARY_STYLE_REUSED_VIA_RULE_NODE);
+            self.flags
+                .remove(ElementDataFlags::PRIMARY_STYLE_REUSED_VIA_RULE_NODE);
         }
         mem::replace(&mut self.styles, new_styles.into())
     }
 
     /// Returns the kind of restyling that we're going to need to do on this
     /// element, based of the stored restyle hint.
-    pub fn restyle_kind(
-        &self,
-        shared_context: &SharedStyleContext
-    ) -> RestyleKind {
+    pub fn restyle_kind(&self, shared_context: &SharedStyleContext) -> RestyleKind {
         if shared_context.traversal_flags.for_animation_only() {
             return self.restyle_kind_for_animation(shared_context);
         }
@@ -343,25 +356,29 @@ impl ElementData {
         }
 
         if self.hint.has_replacements() {
-            debug_assert!(!self.hint.has_animation_hint(),
-                          "Animation only restyle hint should have already processed");
+            debug_assert!(
+                !self.hint.has_animation_hint(),
+                "Animation only restyle hint should have already processed"
+            );
             return RestyleKind::CascadeWithReplacements(self.hint & RestyleHint::replacements());
         }
 
-        debug_assert!(self.hint.has_recascade_self(),
-                      "We definitely need to do something: {:?}!", self.hint);
+        debug_assert!(
+            self.hint.has_recascade_self(),
+            "We definitely need to do something: {:?}!",
+            self.hint
+        );
         return RestyleKind::CascadeOnly;
     }
 
     /// Returns the kind of restyling for animation-only restyle.
-    fn restyle_kind_for_animation(
-        &self,
-        shared_context: &SharedStyleContext,
-    ) -> RestyleKind {
+    fn restyle_kind_for_animation(&self, shared_context: &SharedStyleContext) -> RestyleKind {
         debug_assert!(shared_context.traversal_flags.for_animation_only());
-        debug_assert!(self.has_styles(),
-                      "Unstyled element shouldn't be traversed during \
-                       animation-only traversal");
+        debug_assert!(
+            self.has_styles(),
+            "Unstyled element shouldn't be traversed during \
+             animation-only traversal"
+        );
 
         // return either CascadeWithReplacements or CascadeOnly in case of
         // animation-only restyle. I.e. animation-only restyle never does
@@ -373,26 +390,6 @@ impl ElementData {
         return RestyleKind::CascadeOnly;
     }
 
-    /// Return true if important rules are different.
-    /// We use this to make sure the cascade of off-main thread animations is correct.
-    /// Note: Ignore custom properties for now because we only support opacity and transform
-    ///       properties for animations running on compositor. Actually, we only care about opacity
-    ///       and transform for now, but it's fine to compare all properties and let the user
-    ///       the check which properties do they want.
-    ///       If it costs too much, get_properties_overriding_animations() should return a set
-    ///       containing only opacity and transform properties.
-    pub fn important_rules_are_different(
-        &self,
-        rules: &StrongRuleNode,
-        guards: &StylesheetGuards
-    ) -> bool {
-        debug_assert!(self.has_styles());
-        let (important_rules, _custom) =
-            self.styles.primary().rules().get_properties_overriding_animations(&guards);
-        let (other_important_rules, _custom) = rules.get_properties_overriding_animations(&guards);
-        important_rules != other_important_rules
-    }
-
     /// Drops any restyle state from the element.
     ///
     /// FIXME(bholley): The only caller of this should probably just assert that
@@ -410,16 +407,12 @@ impl ElementData {
         self.flags.remove(ElementDataFlags::WAS_RESTYLED);
     }
 
-    /// Returns whether this element is going to be reconstructed.
-    pub fn reconstructed_self(&self) -> bool {
-        self.damage.contains(RestyleDamage::reconstruct())
-    }
-
     /// Mark this element as restyled, which is useful to know whether we need
     /// to do a post-traversal.
     pub fn set_restyled(&mut self) {
         self.flags.insert(ElementDataFlags::WAS_RESTYLED);
-        self.flags.remove(ElementDataFlags::TRAVERSED_WITHOUT_STYLING);
+        self.flags
+            .remove(ElementDataFlags::TRAVERSED_WITHOUT_STYLING);
     }
 
     /// Returns true if this element was restyled.
@@ -430,13 +423,8 @@ impl ElementData {
 
     /// Mark that we traversed this element without computing any style for it.
     pub fn set_traversed_without_styling(&mut self) {
-        self.flags.insert(ElementDataFlags::TRAVERSED_WITHOUT_STYLING);
-    }
-
-    /// Returns whether the element was traversed without computing any style for
-    /// it.
-    pub fn traversed_without_styling(&self) -> bool {
-        self.flags.contains(ElementDataFlags::TRAVERSED_WITHOUT_STYLING)
+        self.flags
+            .insert(ElementDataFlags::TRAVERSED_WITHOUT_STYLING);
     }
 
     /// Returns whether this element has been part of a restyle.
@@ -464,8 +452,10 @@ impl ElementData {
     /// happens later in the styling pipeline. The former gives us the stronger guarantees
     /// we need for style sharing, the latter does not.
     pub fn safe_for_cousin_sharing(&self) -> bool {
-        !self.flags.intersects(ElementDataFlags::TRAVERSED_WITHOUT_STYLING |
-                               ElementDataFlags::PRIMARY_STYLE_REUSED_VIA_RULE_NODE)
+        !self.flags.intersects(
+            ElementDataFlags::TRAVERSED_WITHOUT_STYLING |
+                ElementDataFlags::PRIMARY_STYLE_REUSED_VIA_RULE_NODE,
+        )
     }
 
     /// Measures memory usage.
