@@ -71,150 +71,6 @@ for (let canonicalId of ["canonical", "canonical-001", "canonical-002"]) {
   });
 }
 
-tests.push({
-  locale: "ru",
-  distribution: "mailru-001",
-  test: engines =>
-    hasParams(engines, "Поиск Mail.Ru", "searchbar", "gp=900201") &&
-    hasParams(engines, "Поиск Mail.Ru", "searchbar", "frc=900201") &&
-    hasDefault(engines, "Поиск Mail.Ru") &&
-    hasEnginesFirst(engines, ["Поиск Mail.Ru"]),
-});
-
-tests.push({
-  locale: "ru",
-  region: "RU",
-  distribution: "mailru-001",
-  test: engines =>
-    hasParams(engines, "Поиск Mail.Ru", "searchbar", "gp=900201") &&
-    hasParams(engines, "Поиск Mail.Ru", "searchbar", "frc=900201") &&
-    hasDefault(engines, "Поиск Mail.Ru") &&
-    hasEnginesFirst(engines, ["Поиск Mail.Ru"]),
-});
-
-tests.push({
-  locale: "az",
-  distribution: "okru-001",
-  test: engines =>
-    hasParams(engines, "Поиск Mail.Ru", "searchbar", "gp=900209") &&
-    hasParams(engines, "Поиск Mail.Ru", "searchbar", "frc=900209") &&
-    hasDefault(engines, "Поиск Mail.Ru") &&
-    hasEnginesFirst(engines, ["Поиск Mail.Ru"]),
-});
-
-tests.push({
-  locale: "en-US",
-  distribution: "okru-001",
-  test: engines =>
-    hasParams(engines, "Поиск Mail.Ru", "searchbar", "gp=900205") &&
-    hasParams(engines, "Поиск Mail.Ru", "searchbar", "frc=900205") &&
-    hasDefault(engines, "Поиск Mail.Ru") &&
-    hasEnginesFirst(engines, ["Поиск Mail.Ru"]),
-});
-
-tests.push({
-  locale: "hy-AM",
-  distribution: "okru-001",
-  test: engines =>
-    hasParams(engines, "Поиск Mail.Ru", "searchbar", "gp=900211") &&
-    hasParams(engines, "Поиск Mail.Ru", "searchbar", "frc=900211") &&
-    hasDefault(engines, "Поиск Mail.Ru") &&
-    hasEnginesFirst(engines, ["Поиск Mail.Ru"]),
-});
-
-tests.push({
-  locale: "kk",
-  distribution: "okru-001",
-  test: engines =>
-    hasParams(engines, "Поиск Mail.Ru", "searchbar", "gp=900206") &&
-    hasParams(engines, "Поиск Mail.Ru", "searchbar", "frc=900206") &&
-    hasDefault(engines, "Поиск Mail.Ru") &&
-    hasEnginesFirst(engines, ["Поиск Mail.Ru"]),
-});
-
-tests.push({
-  locale: "kk",
-  region: "KZ",
-  distribution: "okru-001",
-  test: engines =>
-    hasParams(engines, "Поиск Mail.Ru", "searchbar", "gp=900206") &&
-    hasParams(engines, "Поиск Mail.Ru", "searchbar", "frc=900206") &&
-    hasDefault(engines, "Поиск Mail.Ru") &&
-    hasEnginesFirst(engines, ["Поиск Mail.Ru"]),
-});
-
-tests.push({
-  locale: "ro",
-  distribution: "okru-001",
-  test: engines =>
-    hasParams(engines, "Поиск Mail.Ru", "searchbar", "gp=900207") &&
-    hasParams(engines, "Поиск Mail.Ru", "searchbar", "frc=900207") &&
-    hasDefault(engines, "Поиск Mail.Ru") &&
-    hasEnginesFirst(engines, ["Поиск Mail.Ru"]),
-});
-
-tests.push({
-  locale: "ru",
-  distribution: "okru-001",
-  test: engines =>
-    hasParams(engines, "Поиск Mail.Ru", "searchbar", "gp=900203") &&
-    hasParams(engines, "Поиск Mail.Ru", "searchbar", "frc=900203") &&
-    hasDefault(engines, "Поиск Mail.Ru") &&
-    hasEnginesFirst(engines, ["Поиск Mail.Ru"]),
-});
-
-tests.push({
-  locale: "ru",
-  region: "RU",
-  distribution: "okru-001",
-  test: engines =>
-    hasParams(engines, "Поиск Mail.Ru", "searchbar", "gp=900203") &&
-    hasParams(engines, "Поиск Mail.Ru", "searchbar", "frc=900203") &&
-    hasDefault(engines, "Поиск Mail.Ru") &&
-    hasEnginesFirst(engines, ["Поиск Mail.Ru"]),
-});
-
-tests.push({
-  locale: "tr",
-  distribution: "okru-001",
-  test: engines =>
-    hasParams(engines, "Поиск Mail.Ru", "searchbar", "gp=900210") &&
-    hasParams(engines, "Поиск Mail.Ru", "searchbar", "frc=900210") &&
-    hasDefault(engines, "Поиск Mail.Ru") &&
-    hasEnginesFirst(engines, ["Поиск Mail.Ru"]),
-});
-
-tests.push({
-  locale: "tr",
-  region: "TR",
-  distribution: "okru-001",
-  test: engines =>
-    hasParams(engines, "Поиск Mail.Ru", "searchbar", "gp=900210") &&
-    hasParams(engines, "Поиск Mail.Ru", "searchbar", "frc=900210") &&
-    hasDefault(engines, "Поиск Mail.Ru") &&
-    hasEnginesFirst(engines, ["Поиск Mail.Ru"]),
-});
-
-tests.push({
-  locale: "uk",
-  distribution: "okru-001",
-  test: engines =>
-    hasParams(engines, "Поиск Mail.Ru", "searchbar", "gp=900204") &&
-    hasParams(engines, "Поиск Mail.Ru", "searchbar", "frc=900204") &&
-    hasDefault(engines, "Поиск Mail.Ru") &&
-    hasEnginesFirst(engines, ["Поиск Mail.Ru"]),
-});
-
-tests.push({
-  locale: "uz",
-  distribution: "okru-001",
-  test: engines =>
-    hasParams(engines, "Поиск Mail.Ru", "searchbar", "gp=900208") &&
-    hasParams(engines, "Поиск Mail.Ru", "searchbar", "frc=900208") &&
-    hasDefault(engines, "Поиск Mail.Ru") &&
-    hasEnginesFirst(engines, ["Поиск Mail.Ru"]),
-});
-
 tests.push({
   locale: "zh-CN",
   region: "CN",
@@ -462,89 +318,6 @@ tests.push({
   test: engines => hasDefault(engines, "GMX - Recherche web"),
 });
 
-tests.push({
-  locale: "ru",
-  distribution: "yandex-drp",
-  test: engines =>
-    hasParams(engines, "Яндекс", "searchbar", "clid=2039342") &&
-    // Test that fallback works correct as well.
-    hasParams(engines, "Яндекс", "contextmenu", "clid=2039342") &&
-    hasDefault(engines, "Яндекс") &&
-    hasEnginesFirst(engines, ["Яндекс"]),
-});
-
-tests.push({
-  locale: "ru",
-  distribution: "yandex-planb",
-  test: engines =>
-    hasParams(engines, "Яндекс", "searchbar", "clid=1857376") &&
-    hasDefault(engines, "Яндекс") &&
-    hasEnginesFirst(engines, ["Яндекс"]),
-});
-
-tests.push({
-  locale: "ru",
-  distribution: "yandex-portals",
-  test: engines =>
-    hasParams(engines, "Яндекс", "searchbar", "clid=1923034") &&
-    hasDefault(engines, "Яндекс") &&
-    hasEnginesFirst(engines, ["Яндекс"]),
-});
-
-tests.push({
-  locale: "ru",
-  distribution: "yandex-ru",
-  test: engines =>
-    hasParams(engines, "Яндекс", "searchbar", "clid=1923018") &&
-    hasDefault(engines, "Яндекс") &&
-    hasEnginesFirst(engines, ["Яндекс"]),
-});
-
-tests.push({
-  locale: "tr",
-  distribution: "yandex-tr",
-  test: engines =>
-    hasParams(engines, "Yandex", "searchbar", "clid=1953197") &&
-    hasDefault(engines, "Yandex") &&
-    hasEnginesFirst(engines, ["Yandex"]),
-});
-
-tests.push({
-  locale: "tr",
-  distribution: "yandex-tr-gezginler",
-  test: engines =>
-    hasParams(engines, "Yandex", "searchbar", "clid=1945716") &&
-    hasDefault(engines, "Yandex") &&
-    hasEnginesFirst(engines, ["Yandex"]),
-});
-
-tests.push({
-  locale: "tr",
-  distribution: "yandex-tr-tamindir",
-  test: engines =>
-    hasParams(engines, "Yandex", "searchbar", "clid=1945686") &&
-    hasDefault(engines, "Yandex") &&
-    hasEnginesFirst(engines, ["Yandex"]),
-});
-
-tests.push({
-  locale: "uk",
-  distribution: "yandex-uk",
-  test: engines =>
-    hasParams(engines, "Яндекс", "searchbar", "clid=1923018") &&
-    hasDefault(engines, "Яндекс") &&
-    hasEnginesFirst(engines, ["Яндекс"]),
-});
-
-tests.push({
-  locale: "ru",
-  distribution: "yandex-ru-mz",
-  test: engines =>
-    hasParams(engines, "Яндекс", "searchbar", "clid=2320519") &&
-    hasDefault(engines, "Яндекс") &&
-    hasEnginesFirst(engines, ["Яндекс"]),
-});
-
 function hasURLs(engines, engineName, url, suggestURL) {
   let engine = engines.find(e => e._name === engineName);
   Assert.ok(engine, `Should be able to find ${engineName}`);
