@@ -1021,6 +1021,8 @@ nsresult nsWindowWatcher::OpenWindowInternal(
       SANDBOX_PROPAGATES_TO_AUXILIARY_BROWSING_CONTEXTS) {
     MOZ_ASSERT(windowIsNew, "Should only get here for new windows");
     MOZ_ALWAYS_SUCCEEDS(newBC->SetSandboxFlags(activeDocsSandboxFlags));
+    MOZ_ALWAYS_SUCCEEDS(
+        newBC->SetInitialSandboxFlags(newBC->GetSandboxFlags()));
   }
 
   RefPtr<nsGlobalWindowOuter> win(
@@ -1344,8 +1346,8 @@ nsresult nsWindowWatcher::OpenWindowInternal(
     }
   }
   // If a website opens a popup exit DOM fullscreen
-  if (StaticPrefs::full_screen_api_exit_on_windowOpen() && windowIsNew &&
-      aCalledFromJS && !hasChromeParent && !isCallerChrome && parentWindow) {
+  if (StaticPrefs::full_screen_api_exit_on_windowOpen() && aCalledFromJS &&
+      !hasChromeParent && !isCallerChrome && parentWindow) {
     Document::AsyncExitFullscreen(parentWindow->GetDoc());
   }
 
