@@ -6,8 +6,11 @@ function test() {
   Harness.installsCompletedCallback = finish_test;
   Harness.setup();
 
-  var pm = Services.perms;
-  pm.add(makeURI("http://example.com/"), "install", pm.ALLOW_ACTION);
+  PermissionTestUtils.add(
+    "http://example.com/",
+    "install",
+    Services.perms.ALLOW_ACTION
+  );
 
   var triggers = encodeURIComponent(
     JSON.stringify({
@@ -33,7 +36,7 @@ function download_failed(install) {
 
 function finish_test(count) {
   is(count, 0, "No add-ons should have been installed");
-  Services.perms.remove(makeURI("http://example.com/"), "install");
+  PermissionTestUtils.remove("http://example.com/", "install");
 
   gBrowser.removeCurrentTab();
   Harness.finish();
